@@ -1,7 +1,6 @@
 #!/bin/sh
-#ORIG=yudit-2.6.2
-ORIG=yudit-2.6.2-2
-NEW=yudit-2.6.4.beta1
+ORIG=yudit-2.9.1.beta4
+NEW=yudit-2.9.1.beta5
 diff --exclude=*.kmap --exclude=*.my --exclude=*.mo \
   --exclude=config.status --exclude=config.cache \
   --context=2 --new-file --recursive \
