@@ -43,6 +43,10 @@ SToolBarListener::buttonLeave (void* src, int button)
  */
 SToolBar::SToolBar (const SProperties props) 
 {
+  currentEvent = (int) SD_PARSING_DONE;
+  displayEvent = (int) SD_PARSING_DONE;
+
+  iconUpdater = 0;
   xmanagerCludge = true;
   unsigned int i;
   listener = 0;
@@ -115,6 +119,8 @@ SToolBar::SToolBar (const SProperties props)
        SIconFactory::getIcon("MagnifyPlus"), "Z");
   buttons[SS_MAGNIFY_MINUS] = new SButton (
        SIconFactory::getIcon("MagnifyMinus"), "A");
+  buttons[SS_HIGHLIGHTING] = new SButton (
+       SIconFactory::getIcon("DocumentXX"), "H");
   //b = new SButton (translate ("Cut(&X)"), SIconFactory::getIcon("Cut"));
   unsigned int p=0;
   unsigned int h=0;
@@ -190,14 +196,18 @@ SToolBar::SToolBar (const SProperties props)
   tooltip->setFont(font, fontSize);
   //tooltip->setForeground (fg);
 
-  tooltip->resize (SDimension (p, h));
-  tooltip->move (SLocation (0, -2*(int)h));
   /* when the overall background will be set this will get screwed up a bit. */
   tooltip->setBackground (SColor("yellow").lighter());
 
   add (tooltip);
+  tooltip->resize (SDimension (p, h));
+  tooltip->move (SLocation (0, -2*(int)h));
+
   add (title);
   add (titleEncoding);
+  iconUpdater = new SIconUpdater (window, title, 
+        &currentEvent, &displayEvent);
+  CHECK_NEW (iconUpdater);
 }
 
 /**
@@ -263,9 +273,31 @@ SToolBar::buildDisplayInputs()
 
 SToolBar::~SToolBar ()
 {
+  if (iconUpdater) delete iconUpdater;
 }
+
 void
-SToolBar::setFileName (const SString& _filename, const SString& _encoding)
+SToolBar::setHighlightName (const SString& _highlightName)
+{
+  // display only the last part.
+  SStringVector v (_highlightName, ":", true);
+  SString f = "";
+  if (v.size() == 2)
+  {
+    f.append (v[0]);
+    f.append (":");
+    f.append (v[1]);
+  }
+  else if (v.size() == 1)
+  {
+    f.append (v[0]);
+  }
+  highlightName = f;
+  
+}
+
+void
+SToolBar::setFileName (const SString& _filename, const SString& _encoding) 
 {
   window->redraw (true, title->getLocation().x, 
      title->getLocation().y, title->getSize().width, title->getSize().height);
@@ -277,6 +309,7 @@ SToolBar::setFileName (const SString& _filename, const SString& _encoding)
   fn.append  (_encoding);
   //fn.append ("]");
   titleEncoding->setText (fn);
+  
   title->setText (_filename);
   SDimension td = title->getPreferredSize();
   SDimension ted = titleEncoding->getPreferredSize();
@@ -506,6 +539,13 @@ SToolBar::enterWindow (void* source)
       case SS_YIELD_EMBEDDING: str = translate ("__Y__ield Embedding"); break;
       case SS_MAGNIFY_MINUS: str = translate ("Smaller(__A__)"); break;
       case SS_MAGNIFY_PLUS: str = translate ("Bigger(__Z__)"); break;
+      case SS_HIGHLIGHTING: str = translate ("__H__ighlighting"); 
+        if (highlightName != "")
+        {
+          str.append (" ");
+          str.append (highlightName);
+        }
+        break;
       case SS_FONT: str = translate ("Font");
         max = (int)fonts.size();
         break;
@@ -816,3 +856,63 @@ SToolBar::directionChanged (bool lr)
   currentLR = lr;
 
 }
+
+void
+SToolBar::syntaxChanged (SS_EventType _evnt)
+{
+  currentEvent = (int)_evnt;
+  if (currentEvent != displayEvent) iconUpdater->start ();
+}
+
+SIconUpdater::SIconUpdater (SWindow* _w, SOpaqueLabel* _label, 
+   int* _currentEvent, int* _displayEvent)
+{
+  label = _label;
+  window = _w;
+  currentEvent = _currentEvent;
+  displayEvent = _displayEvent;
+  eventTimer = 0;
+}
+
+SIconUpdater::~SIconUpdater ()
+{
+  if (eventTimer) delete eventTimer;
+}
+
+void
+SIconUpdater::start ()
+{
+  if (eventTimer) return;
+  eventTimer = STimer::newTimer(1000, this);
+}
+
+bool
+SIconUpdater::timeout (const SEventSource* s)
+{
+  delete eventTimer;
+  eventTimer = 0;
+  if (*displayEvent == *currentEvent)
+  {
+    return false;
+  }
+  *displayEvent = *currentEvent;
+  SSyntaxListener::SS_EventType _evnt 
+       = (SSyntaxListener::SS_EventType) *displayEvent;
+  switch (_evnt)
+  {
+  case SSyntaxListener::SD_PARSING_STARTED:
+    label->setIcon (SIconFactory::getIcon("DocumentXX"));
+    break;
+  case SSyntaxListener::SD_PARSING_DONE:
+    label->setIcon (SIconFactory::getIcon("Document"));
+    break;
+  default:
+    label->setIcon (SIconFactory::getIcon("DocumentCX"));
+    break;
+  }
+  window->redraw (true, label->getLocation().x, 
+     label->getLocation().y, label->getSize().width, 
+     label->getSize().height);
+
+  return false;
+}
