@@ -26,6 +26,23 @@
 #include "swidget/SLabel.h"
 #include "stoolkit/SBinVector.h"
 #include "stoolkit/SCharClass.h"
+#include "stoolkit/SEvent.h"
+
+class SIconUpdater : public SEventTarget
+{
+public:
+  SIconUpdater (SWindow* _w, SOpaqueLabel* label, 
+       int* currentEvent, int* displayEvent);
+  ~SIconUpdater();
+  bool timeout (const SEventSource* s);
+  void start ();
+private:
+  int*           currentEvent; 
+  int*           displayEvent; 
+  STimer*        eventTimer;
+  SOpaqueLabel*  label;
+  SWindow*       window;
+};
 
 /**
  * A caret that redraws itself differently for lr and rl text 
@@ -40,13 +57,14 @@ public:
   virtual void buttonLeave (void* src, int button);
 };
 
-class SToolBar  : public SPanel, public SButtonListener
+class SToolBar  : public SPanel, public SButtonListener, public SSyntaxListener
 {
 public:
   enum SButtonIndex {SS_OPEN=0, 
    SS_SAVE, SS_PRINT, SS_PRINT_PREVIEW, SS_FIND, SS_GOTO, 
    SS_UNDO, SS_REDO, SS_DOCUMENT_EMBEDDING, SS_SET_OVERRIDE, SS_SET_EMBEDDING,
    SS_YIELD_EMBEDDING, SS_MAGNIFY_PLUS, SS_MAGNIFY_MINUS, 
+   SS_HIGHLIGHTING,
    SS_FONT, SS_INPUT, SS_PARAGRAPH_BREAK, SS_MAX };
 
   SToolBar (const SProperties props);
@@ -54,7 +72,8 @@ public:
 
   SButton* buttons[SS_MAX];
   void setListener (SToolBarListener* listener);
-  void setFileName (const SString& fileName, const SString& encoding);
+  void setFileName (const SString& fileName, const SString& encoding); 
+  void setHighlightName (const SString& _fileType);
   void inputChanged (const SStringVector& _inputs, unsigned int _current);
 
   virtual void buttonPressed (void* source, const SAccelerator* accel);
@@ -63,6 +82,9 @@ public:
   virtual void redraw(SWindow* w, int x, int y,
      unsigned int width ,unsigned int height);
   virtual void resize(const SDimension & d);
+  // SSyntaxListener
+  virtual void syntaxChanged (SS_EventType _evnt);
+
   unsigned int  currentFontSize;
   SStringVector fontsizes;
   unsigned int  currentInput;
@@ -98,6 +120,8 @@ private:
   SOpaqueLabel*     titleEncoding;
   SLabel*           tooltip;
   SStringVector     displayInputs;
+  SIconUpdater*     iconUpdater;
+  SString           highlightName;
 
   void setFont (int index);
   void setFontSize (int index);
@@ -111,6 +135,8 @@ private:
   unsigned int preferredWidth;
   SToolBarListener* listener;
   bool  xmanagerCludge;
+  int   currentEvent;
+  int   displayEvent;
 };
 
 #endif /* SToolBar_h */
