@@ -29,14 +29,40 @@
 #include <swindow/SScriptProcessor.h>
 #include <stdlib.h>
 
-static int translateHighlightMode (const SString& hl);
+static SString translateHighlightMode (const SString& hl);
+static SString getPrintHelp ();
 static int getLineNumber (const SString& s);
 
+#define SGC_DOUBLE_BUFFER  true
+
 /**
  * A caret that redraws itself differently for lr and rl text 
  */
 SYudit::SYudit (const SStringVector& args, const SProperties& props)
 {
+  // No use to make this main window double buffered 
+  // because it almost only redraws its borders and that's it 
+// This is done in SFrame.
+//  setThisDoubleBuffered (false);
+  SPanel::setAllDoubleBuffered (SGC_DOUBLE_BUFFER);
+  // preparse args
+  unsigned int i;
+  for (i=1; i<args.size(); i++)
+  {
+    if (args[i] == SString("--"))
+    {
+      break;
+    }
+    else if (args[i] == SString("-ndb"))
+    {
+      SPanel::setAllDoubleBuffered (false);
+    }
+    else if (args[i] == SString("-db"))
+    {
+      SPanel::setAllDoubleBuffered (true);
+    }
+  }
+
   freeHandShown = false;
   freehand = 0;
   freeHandHeight = 0;
@@ -181,6 +207,8 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
   
 
   editor = new STextEdit ();
+  editor->addSyntaxListener (toolbar);
+
   currentFontSize.append ((char)0);
   double df;
   if (sscanf (currentFontSize.array(), "%lf", &df)==0 || df < 2.0) df = 2.0;
@@ -229,7 +257,7 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
   }
   else
   {
-    fprintf (stderr, "Yudit: unkown yudit.editor.xinputs.style=%*.*s\n",
+    fprintf (stderr, "Yudit: unknown yudit.editor.xinputs.style=%*.*s\n",
        SSARGS(inputStyle));
   }
 
@@ -262,11 +290,37 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
   {
     editor->setWordWrap (false);
   }
+  SSyntaxColors syn;
+  syn.colors[(unsigned int) SSyntax::SD_NONE] = SColor ("cyan");
+/*
+  got rid of none. For errors and none we use the default fg.
+    = props.getProperty ("yudit.editor.syntax.none.foreground", "green");
+*/
+
+  syn.colors[(unsigned int) SSyntax::SD_ERROR] 
+    = props.getProperty ("yudit.editor.syntax.error.foreground", "OrangeRed3");
+  syn.colors[(unsigned int) SSyntax::SD_NUMBER] 
+    = props.getProperty ("yudit.editor.syntax.number.foreground", "orange");
+  syn.colors[(unsigned int) SSyntax::SD_STRING] 
+    = props.getProperty ("yudit.editor.syntax.string.foreground", "magenta");
+  syn.colors[(unsigned int) SSyntax::SD_COMMENT] 
+    = props.getProperty ("yudit.editor.syntax.comment.foreground", "lightgray");
+  syn.colors[(unsigned int) SSyntax::SD_KEYWORD] 
+    = props.getProperty ("yudit.editor.syntax.token.foreground", "yellow");
+  syn.colors[(unsigned int) SSyntax::SD_VARIABLE] 
+    = props.getProperty ("yudit.editor.syntax.variable.foreground", "red");
+  syn.colors[(unsigned int) SSyntax::SD_DEFINE] 
+    = props.getProperty ("yudit.editor.syntax.define.foreground", "cyan");
+  syn.colors[(unsigned int) SSyntax::SD_CONTROL] 
+    = props.getProperty ("yudit.editor.syntax.control.foreground", "CornflowerBlue");
+
+  editor->setSyntaxColors (syn);
+
   const SString& hl = props["yudit.editor.syntax"];
-  int mode = translateHighlightMode (hl);
-  if (mode >= 0) 
+  SString mode = translateHighlightMode (hl);
+  if (mode.size() >= 0) 
   {
-    editor->setHighlightMode ((STextView::SS_HighlightMode)mode);
+    editor->setSyntax (mode);
   }
 
   SEditor e(currentParagraphSeparator);
@@ -297,8 +351,10 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
   SDimension pw = toolbar->getPreferredSize();
   /* get the dimension */
   SDimension ms(props["yudit.default.geometry"]);
-  if (ms.width < pw.width + 10) ms.width = pw.width + 10;
-  if (ms.height < pw.height + 10) ms.width = pw.height + 10;
+  //if (ms.width < pw.width + 10) ms.width = pw.width + 10;
+  //if (ms.height < pw.height + 10) ms.width = pw.height + 10;
+  if (ms.width <  150) ms.width = 150;
+  if (ms.height < 100) ms.height =  100;
   resize (ms);
   setMinimumSize (SDimension (150, 100));
 
@@ -312,6 +368,7 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
 
   kmapDialog = 0;
   fileDialog = 0;
+  highlightDialog = 0;
   textDialog = 0;
 
 #ifdef USE_WINAPI
@@ -320,7 +377,6 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
   SToolBar::SFormatIndex ft=SToolBar::SS_FORMAT_UNIX;
 #endif
   toolbar->setParagraphSeparator (ft, true);
-
   int lineNum = -1;
   if (args.size())
   {
@@ -353,6 +409,10 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
       {
         help = true;
       }
+      else if (args[i] == SString("-ndb") || args[i] == SString("-db"))
+      {
+        // already processed
+      }
       else if (args[i] == SString("-us"))
       {
         SScriptProcessor::support(true);
@@ -369,8 +429,10 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
 	      && i+1 < args.size()) {
         i++;
 	ms = SDimension (args[i]);  
-        if (ms.width < pw.width + 10) ms.width = pw.width + 10;
-        if (ms.height < pw.height + 10) ms.width = pw.height + 10;
+        //if (ms.width < pw.width + 10) ms.width = pw.width + 10;
+        //if (ms.height < pw.height + 10) ms.width = pw.height + 10;
+        if (ms.width < 150) ms.width =  150;
+        if (ms.height < 100) ms.width =  100;
         resize (ms);
       }
       else 
@@ -402,7 +464,9 @@ SYudit::SYudit (const SStringVector& args, const SProperties& props)
     load (v, filename);
   }
 
-  toolbar->setFileName(currentFileName, lastencoding);
+  toolbar->setFileName(currentFileName, lastencoding); 
+  toolbar->setHighlightName (editor->getHighlightName());
+
   setFileTitle ();
   SIcon* icon = SIconFactory::getIcon("Yudit");
   setApplicationImage (icon->getImage ());
@@ -602,9 +666,7 @@ SYudit::buttonPressed (void* src, int which, bool accel)
     command->setFocus();
     break;
   case SToolBar::SS_PRINT:
-    messagelabel->setMessage (
-      translate ("usage: print [-o file] [-p printer] [-e exec] [-break] [-hsize header-font-size]"), 
-       SMessageLabel::SS_INFO);
+    messagelabel->setMessage (getPrintHelp (), SMessageLabel::SS_INFO);
     pstr.append ("print ");
     pstr.append (lastprinterOption);
     command->setText(pstr);
@@ -732,11 +794,77 @@ SYudit::buttonPressed (void* src, int which, bool accel)
   case SToolBar::SS_PARAGRAPH_BREAK: 
     editor->setParagraphSeparator (currentParagraphSeparator, true);
     break;
+  case SToolBar::SS_HIGHLIGHTING:
+    if (!accel)
+    {
+       createHighlightDialog();
+       SString gotMode = editor->getHighlightName();
+       highlightDialog->setHighlightName (gotMode);
+       ok = highlightDialog->getInput ("Yudit: Highlighting");
+       if (!ok)
+       {
+          return;
+       }
+       SString hlmode = highlightDialog->getHighlightName();
+       pstr.clear();
+       pstr.append ("syntax ");
+       pstr.append (" ");
+       //pstr.append (quoteFileName (hlmode));
+       pstr.append (hlmode);
+       //command->setText (pstr);
+       //command->setCursorIndex (SCursorIndex(0,10000));
+       if (setHighlight (hlmode))
+       {
+         command->putHistory(pstr);
+       }
+       return;
+    }
+    else
+    {
+      SString gotMode = editor->getHighlightName();
+      SString cmd ("syntax");
+      cmd.append (" ");
+      cmd.append (gotMode);
+      command->setText (cmd);
+      command->setCursorIndex (SCursorIndex(0,10000));
+      command->setFocus();
+    }
+    break;
   default:
     break;
   }
 }
 
+bool
+SYudit::setHighlight (const SString& syntaxMode)
+{
+  SString smode;
+  if ((smode = translateHighlightMode (syntaxMode)) == "")
+  {
+    SString err = translate ("usage: ");
+    err.append ("syntax none|simple|simple-dark|hunspell:en_US|...");
+    messagelabel->setMessage (err, SMessageLabel::SS_ERR);
+    return false;
+  }
+  SString oldM = editor->getHighlightName ();
+  editor->setSyntax (smode);
+  SString gotMode = editor->getSyntaxName();
+  if (gotMode != smode && smode != "simple" 
+      && smode != "simple-dark" && smode != "none")
+  {
+    SString err = translate ("Can not set syntax: ");
+    err.append (smode);
+    messagelabel->setMessage (err, SMessageLabel::SS_ERR);
+    editor->setSyntax (oldM);
+    return false;
+  }
+  // dont display these
+  toolbar->setHighlightName (editor->getHighlightName());
+
+  userProps.put ("yudit.editor.syntax", smode);
+  return true;
+}
+
 void
 SYudit::buttonLeave (void* src, int which)
 {
@@ -807,8 +935,8 @@ SYudit::textEntered (void *source)
     print (cv, text);
     return;
   }
-  if (cv.size()>0 && (cv[0] == "help" || cv[0] == "test" || 
-       helpStrings.get(cv[0]) != 0) 
+  if ((cv.size()>0 && (cv[0] == "help" || cv[0] == "test" || 
+       helpStrings.get(cv[0]) != 0)) 
      || (cv.size() > 1 && cv[0] == "howto"))
   {
     load (cv, text);
@@ -846,18 +974,19 @@ SYudit::textEntered (void *source)
   }
   if (cv.size() > 0 && cv[0] == "syntax")
   {
-    int smode = -1;
-    if (cv.size() != 2 || (smode = translateHighlightMode (cv[1])) < 0)
+    SString smode;
+    if (cv.size() != 2)
     {
       SString err = translate ("usage: ");
-      err.append ("syntax none|simple|simple-dark");
+      err.append ("syntax none|simple|simple-dark|hunspell:en_US... ");
       messagelabel->setMessage (err, SMessageLabel::SS_ERR);
       return;
     }
-    editor->setHighlightMode ((STextView::SS_HighlightMode)smode);
-    userProps.put ("yudit.editor.syntax", cv[1]);
-    editor->setFocus();
-    command->putHistory(text);
+    if (setHighlight (cv[1]))
+    {
+      editor->setFocus();
+      command->putHistory(text);
+    }
     return;
   }
   if (cv.size() > 0 && cv[0] == "wordwrap")
@@ -905,6 +1034,7 @@ SYudit::print (const SStringVector& args, const SString& str)
   unsigned int i;
   bool shownl = false;
   bool islpr = false;
+  double fontSize = messagebar->fontSize;
 
   SPrinter::SMedia media = SPrinter::A4;
   SPrinter::SOrientation orientation = SPrinter::PORTRAIT;
@@ -921,6 +1051,21 @@ SYudit::print (const SStringVector& args, const SString& str)
       i++;
       hsize = args[i];
     }
+    else if ((args[i] == "-size") && i+1 < args.size())
+    {
+      i++;
+      SString sarg = args[i];
+      sarg.append ((char)0);
+      sscanf (sarg.array(), "%lf", &fontSize);
+      if (fontSize > 1000.0)
+      {
+        fontSize = 1000.0;
+      }
+      if (fontSize < 1.0)
+      {
+        fontSize = 1.0;
+      }
+    }
     else if ((args[i] == "-out" || args[i] =="-o") && i+1 < args.size())
     {
       i++;
@@ -1063,9 +1208,7 @@ SYudit::print (const SStringVector& args, const SString& str)
   if (i<args.size() || (filename.size()!=0 && executable.size()!=0)
      || (filename.size()!=0 && printer.size() > 0))
   {
-    messagelabel->setMessage (
-      translate ("usage: print [-o file] [-p printer] [-e exec] [-break] [-hsize header-font-size]"), 
-      SMessageLabel::SS_ERR);
+    messagelabel->setMessage (getPrintHelp(), SMessageLabel::SS_ERR);
     return;
   }
   SOutputStream os;
@@ -1100,7 +1243,7 @@ SYudit::print (const SStringVector& args, const SString& str)
     {
       SString ems ("Can not open: ");
       ems.append (SS_LB_LRE);
-      ems.append ("");
+      ems.append ("'");
       ems.append (filename);
       ems.append ("'");
       ems.append (SS_LB_PDF);
@@ -1111,9 +1254,18 @@ SYudit::print (const SStringVector& args, const SString& str)
   SString fontName = toolbar->fonts[toolbar->currentFont];
   
   SPrinter p (writer, SPrinter::POSTSCRIPT, media, orientation);
-  SUniPrint uniPrint(p, fontName, hsized, fontName, messagebar->fontSize);
+  SUniPrint uniPrint(p, fontName, hsized, fontName, fontSize);
 
-  SString text = editor->getText();
+  SString text = editor->getSelectedText();
+  SString sel = currentFileName;
+  if (text.size() == 0)
+  { 
+    text = editor->getText();
+  }
+  else
+  {
+    sel.append (" (excerpt)");
+  }
 
   uniPrint.setDocumentEmbedding (toolbar->currentEmbedding);
   uniPrint.setLineEndMark(shownl);
@@ -1126,7 +1278,7 @@ SYudit::print (const SStringVector& args, const SString& str)
   messagelabel->setMessage (
      translate ("Printing..."),
       SMessageLabel::SS_NONE);
-  if (!uniPrint.print (currentFileName, text, true))
+  if (!uniPrint.print (sel, text, true))
   {
       toolbar->setPrinting (false);
       SString ems ("Printing failed on '");
@@ -1145,7 +1297,7 @@ SYudit::print (const SStringVector& args, const SString& str)
     sprintf (a, tr.array(), pagecount);
     if (uniPrint.hasNative())
     {
-      SString ms ("Bitmap font (Bad Quality). ");
+      SString ms ("Bitmap font printed (bad printing quality).");
       ms.append (a);
       messagelabel->setMessage (ms, SMessageLabel::SS_WARN);
     }
@@ -1336,7 +1488,7 @@ SYudit::load (const SStringVector& args, const SString& str)
       lastencoding = encode;
       command->clear();
       currentFileName = filename;
-      toolbar->setFileName(currentFileName, lastencoding);
+      toolbar->setFileName(currentFileName, lastencoding); 
       setFileTitle ();
 
       editor->clear();
@@ -1784,6 +1936,11 @@ SYudit::saveProperties ()
   userProps.put ("yudit.editor.input", s);
   s = toolbar->fontsizes[toolbar->currentFontSize];
   userProps.put ("yudit.editor.fontsize", s);
+  s = "";
+  s.print ((unsigned int) getSize().width);
+  s.append ("x");
+  s.print ((unsigned int) getSize().height);
+  userProps.put ("yudit.default.geometry", s);
   if (freehand)
   {
     SString s = freehand->getConverter();
@@ -1822,6 +1979,7 @@ SYudit::saveProperties ()
   full.merge (current);
   /* merge in changes */
   full.merge (changes); 
+  full.put ("00HEADER.version.yudit", SD_YUDIT_VERSION);
 
   /* That stupid old properties header comment. */
   const SString *pstr = full.get ("yudit.background.#");
@@ -1859,6 +2017,25 @@ SYudit::dropped (void* p, const SString& mimetype, const SString& data)
   SString file=v[0];
   if (mimetype == "text/uri-list")
   {
+    // FIX %XX
+    SString ret;
+    char* c = 0;
+    char buff[3];
+    for (unsigned i=0; i<file.size(); i++) 
+    {
+      if (file[i] == '%' && i+2<file.size()) 
+      {
+        buff[0] = file[i+1];
+        buff[1] = file[i+2];
+        buff[2] = 0;
+        c = 0;
+        ret.append ((char)(unsigned char)strtol (buff, &c, 16));
+        i=i+2;
+      } else {
+        ret.append ((char) file[i]);
+      }
+    }
+    file = ret;
     if (file.size() < strlen ("file:") 
        || strncmp (file.array(), "file:", strlen ("file:"))!=0)
     {
@@ -1886,7 +2063,7 @@ SYudit::dropped (void* p, const SString& mimetype, const SString& data)
   {
     messagelabel->setMessage (translate ("Unsaved changes. Enter discards!"), SMessageLabel::SS_ERR);
     cm.append ("-yes ");
-    cm.append (file);
+    cm.append (quoteFileName (file));
     command->setText(cm);
     command->setCursorIndex (SCursorIndex(0,10000));
     command->setFocus();
@@ -1895,7 +2072,7 @@ SYudit::dropped (void* p, const SString& mimetype, const SString& data)
   {
     messagelabel->setMessage (translate ("open -yes -e encoding file"), 
        SMessageLabel::SS_INFO);
-    cm.append (file);
+    cm.append (quoteFileName (file));
     command->setText(cm);
     command->setCursorIndex (SCursorIndex(0,10000));
     command->putHistory(cm);
@@ -2010,6 +2187,31 @@ SYudit::createFileDialog () // If needed
   fileDialog->setApplicationImage (icon->getImage ());
 }
 
+void
+SYudit::createHighlightDialog () // If needed
+{
+  if (highlightDialog) return;
+
+  highlightDialog = new SHighlightD();
+  highlightDialog->setModal (this);
+  highlightDialog->setBackground (originalProperties["yudit.background"]);
+  highlightDialog->setLabelForeground (originalProperties["yudit.label.foreground"]);
+  highlightDialog->setTitleForeground (originalProperties["yudit.title.foreground"]);
+  highlightDialog->setSliderBackground (originalProperties["yudit.slider.background"]);
+
+  /* set the filetypes */
+  SString font = originalProperties["yudit.default.font"];
+  SString fontSizeString=originalProperties["yudit.default.fontsize"];
+  double fontSize=0;
+  fontSizeString.append ((char)0);
+  sscanf (fontSizeString.array(), "%lf", &fontSize);
+  if (fontSize < 2.0) fontSize = 2.0;
+  highlightDialog->setFont (font, fontSize);
+
+  SIcon* icon = SIconFactory::getIcon("Yudit");
+  highlightDialog->setApplicationImage (icon->getImage ());
+}
+
 void
 SYudit::createKMapDialog ()
 {
@@ -2294,26 +2496,15 @@ SYudit::setFileTitle ()
  * Translate our string highlight mode to an integer
  * @return -1 if it can not be translated.
  */
-static int 
+static SString 
 translateHighlightMode (const SString& hl)
 {
-  if (hl == "none")
+  if (hl == "none" || hl == "simple" || hl == "simple-dark")
   {
-    return (int) STextView::HM_None;
+    return SString(hl);
   }
-  else if (hl == "simple")
-  {
-    return (int) STextView::HM_Simple;
-  }
-  else if (hl == "simple-dark")
-  {
-    return (int) STextView::HM_SimpleDark;
-  }
-  else if (hl == "html")
-  {
-    return (int) STextView::HM_HTML;
-  }
-  return -1;
+  if (SSyntax::isSupported (hl)) return SString(hl);
+  return SString("");
 }
 
 /**
@@ -2342,3 +2533,11 @@ getLineNumber (const SString& s)
   } 
   return line;
 }
+
+static SString getPrintHelp ()
+{
+   SString pstr = 
+     translate ("usage: print [-o file] [-p printer] [-e exec] [-break] [-hsize header-font-size]"); 
+  pstr.append (" [-m A4] [-L]");
+  return SString (pstr);
+}
