@@ -581,7 +581,7 @@ main (int argc, char* argv[])
       fprintf (stdout, "%d.name...............: [%s]\n", i, nameb);
       map.getComment (commentb, 1024, i);
       fprintf (stdout, "%d.comment............: [%s]\n", i, commentb);
-      type = "unkown";
+      type = "unknown";
       if (mp->itemType == SBMapItem::SBMapNToN)
         type = "my";
       if (mp->itemType == SBMapItem::SBMapBumap)
@@ -656,8 +656,11 @@ main (int argc, char* argv[])
     
     int index, in_bytesize, out_bytesize;
     int len, in_bytesize_orig, out_bytesize_orig;
-
-    fscanf (stdin, "%s", ce);
+    ce[255] = 0;
+    if (fscanf (stdin, "%255s", ce) == 0) {
+      fprintf (stderr,  "Can not parse line.\n");
+      return (1);
+    } 
     index = 0;
     if (ce[0] == 'd')
     {
@@ -688,8 +691,14 @@ main (int argc, char* argv[])
     {
       index = atoi(ce);
     }
-    fscanf (stdin, "%d", &in_bytesize_orig);
-    fscanf (stdin, "%d", &out_bytesize_orig);
+    if (fscanf (stdin, "%d", &in_bytesize_orig) == 0) {
+      fprintf (stderr,  "Can not parse line.\n");
+      return (1);
+    }
+    if (fscanf (stdin, "%d", &out_bytesize_orig) == 0) {
+      fprintf (stderr,  "Can not parse line.\n");
+      return (1);
+    }
 
     switch (in_bytesize_orig)
     {
@@ -732,7 +741,11 @@ main (int argc, char* argv[])
         fprintf (stdout,  "Enter hex bytestring nibbles: ");
       }
       fflush (stdout);
-      fscanf (stdin, "%s", in);
+      in[1023] = 0;
+      if (fscanf (stdin, "%1023s", in) == 0) {
+        fprintf (stdout,  "Can not parse line");
+        return (1);
+      }
       if (in_bytesize_orig==1)
       {
         len = strlen (in);
