@@ -324,7 +324,9 @@ SBHashtable::remove (const SString& index)
         }
         if (j==1)
         {
-           delete bv; GET_BUCKET (buffer->array, mod) = 0;
+           delete [] bv;
+	       bv = NULL;
+	       GET_BUCKET (buffer->array, mod) = 0;
            break;
         }
         break;
@@ -365,7 +367,8 @@ SBHashtable::cleanup ()
              bi = bv[j];
              delete bi;
           }
-          delete bv;
+          delete [] bv;
+	  bv = NULL;
        }
     }
     delete buffer;
@@ -532,7 +535,8 @@ _subBucketAppend (SBucketVector bv, SBucketItem* bi)
    }
    if (bv != 0)
    {
-     delete bv;
+     delete [] bv;
+     bv = NULL;
    }
    nv[count] = bi;
    nv[count+1] = 0;
