@@ -1,10 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 2003  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -951,10 +948,8 @@ SBMap::encode (int mapIndex, const void *in, int in_size,
 {
   unsigned int  i;
   unsigned int  j;
-  int  reject;
   SBMapItem::SFound  result;
 
-  reject = SS_REJECT;
   if (status!=true || maps ==0 || mapIndex >= mapSize) return SS_REJECT;
 
   SBMapItem* map = maps[mapIndex];
@@ -3786,7 +3781,7 @@ SBMapItem::getDecoderMap (SStringVector* key, SStringVector* value,
      SV_UCS4 ucs4v;
      for (unsigned int i=0; i<vlesize; i=i+(1<<outWordSize))
      {
-       SS_UCS4 ucs4;
+       SS_UCS4 ucs4 = 0;
        switch (outWordSize)
        {
        case 0:
