@@ -1,10 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 2003  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -22,7 +19,7 @@
  
 /**
  * Generic map (n to n)
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 1999-12-04
  */
 #ifndef SBMap_H
@@ -170,7 +167,7 @@ public:
 
   void buildStateMachine ();
   int  serialize (SString* fd, SFileFormat format, int last=0);
-  int SBMapItem::getSerializeSize ();
+  int getSerializeSize ();
 
 
   // Get rid of the state machine.
@@ -317,8 +314,8 @@ public:
     return (unsigned int) maps[mapIndex]->outWordSize;
   }
 
-  bool SBMap::makeUnicodeMap ();
-  bool SBMap::makeStraightMap ();
+  bool makeUnicodeMap ();
+  bool makeStraightMap ();
 
   // Return max index
   int getSize (int index=-1);
