@@ -19,6 +19,7 @@
 
 #include "stoolkit/SCharClass.h" 
 #include "stoolkit/SUniMap.h"
+#include "stoolkit/SBinHashtable.h"
 
 const char* ssCharClass[] = {
   "Xx",   // 0 unknown
@@ -95,11 +96,22 @@ getCharClass(SS_UCS4 in)
     CHECK_NEW (ccMap);
   } 
   if (!ccMap->isOK()) return SD_CC_Xx;
+  SString key((char*)&in, sizeof (in));
   // Bengali clusters may start with a virama...
   if (in==0x09cd) return SD_CC_Mc;
+  // BLISSYMBOLICS65
+  if (in >= 0xe010 && in <= 0xe019) return SD_CC_Nd;
+  // ROVASIRAS
+  if (in >= 0xee31 && in <= 0xee3f) return SD_CC_Nd;
+  // ROVASIRAS
+  if (in ==  0xee2f) return SD_CC_Zs;
+  // BLISSYMBOLICS65
+  if (in >= 0xe000 && in <=0xe0af) return SD_CC_Lo;
+  // ROVASIRAS
+  if (in >= 0xee00 && in <=0xee8b) return SD_CC_Lo;
+
   unsigned int kindex = ccMap->getEncodePosition (in);
-  SS_UCS4 vle = ccMap->getEncodeValue(kindex);
-  return (SD_CharClass) vle;
+  return (SD_CharClass) ccMap->getEncodeValue(kindex);
 }
 
 SD_BiDiClass
