@@ -101,6 +101,7 @@ typedef enum
 
 #define SD_CD_CTRL 0
 #define SD_CD_LF ((SS_UCS4)'\n')
+#define SD_CD_FF ((SS_UCS4)'\f')
 #define SD_CD_CR ((SS_UCS4)'\r')
 #define SD_CD_TAB ((SS_UCS4)'\t')
 #define SD_CD_LS 0x2028 /* line separator */
@@ -124,6 +125,7 @@ typedef enum
 #define SS_LB_UNIX "\n"
 #define SS_LB_LS "\342\200\250"
 #define SS_LB_PS "\342\200\251" /* PARAGRAPH BREAKING */
+#define SS_LB_FF "\f"
 
 #define SS_LB_LRO "\342\200\255"
 #define SS_LB_RLO "\342\200\256"
@@ -135,8 +137,8 @@ typedef enum
  * These line breaking chars are not supported here now.
  */
 #define SS_LB_NEL "\702\102"
-#define SS_LB_VT "\013" /* PARAGRAPH BREAKING */
-#define SS_LB_FF "\014" /* PARAGRAPH BREAKING */
+#define SS_LB_P_VT "\013" /* PARAGRAPH BREAKING */
+#define SS_LB_P_FF "\014" /* PARAGRAPH BREAKING */
 
 
 extern const char* ssCharClass[SD_CC_MAX];
