@@ -253,7 +253,7 @@ SGlyph::SGlyph (const SV_UCS4 &decomp, SS_UCS4 comp,
     shared->mirror = precomp ? getMirroredCharacter(precomp) : 0;
     shared->tab = (fchar == SD_CD_TAB);
     shared->lineend = (fchar==SD_CD_LF  || fchar==SD_CD_CR 
-           || fchar==SD_CD_PS || fchar==SD_CD_LS);
+           || fchar==SD_CD_PS || fchar==SD_CD_LS || fchar==SD_CD_FF);
 
     /* clusters and stuff need shared->type and shared->bidi */
     if (cluster>0)
@@ -635,7 +635,6 @@ SGlyph::isWhiteSpace() const
 bool
 SGlyph::isNumber() const
 {
-  if (isSpecial ()) return true;
   SD_CharClass type = getType();
   switch (type)
   {
@@ -649,9 +648,32 @@ SGlyph::isNumber() const
   return false;
 }
 
+/**
+ * This method is used to higlight letters with a color.
+ */
+bool
+SGlyph::isLetter() const
+{
+  SD_CharClass type = getType();
+  switch (type)
+  {
+  case SD_CC_Lu:
+  case SD_CC_Ll:
+  case SD_CC_Lt:
+  case SD_CC_Lm:
+  case SD_CC_Lo:
+    return true;
+  default:
+    break;
+  }
+  return false;
+}
+
 
 /**
  * Check if this is a valid delimiter
+ * This method is used to facilitate wordwarp functionality.
+ *
  * Supports many of the delimiters in unicode, including CJK delimiters, arabic, hebrew and greek...
 * Addition by Maarten van Gompel <proycon@anaproy.homeip.net>
  */
@@ -679,6 +701,45 @@ SGlyph::isDelimiter() const
   return false;
 }
 
+// previously we used isDelimiter to decide weather the word can be broken
+// at this poing. Now I added CJK and KANA because they can wrap any time.
+bool
+SGlyph::canWrap() const
+{
+  if (isDelimiter ()) return true;
+  SS_UCS4 chr = shared->precomposed;
+  // CJK Symbols and Punctuation
+  if (chr >= 0x3000 && chr <=0x303f) return true;
+  // Hiragana
+  if (chr >= 0x3040 && chr <=0x309f) return true;
+  // Katakana
+  if (chr >= 0x30A0 && chr <=0x30ff) return true;
+  // Bopomofo
+  // Hangul Compatibility Jamo
+  // CJK Strokes 
+  // Katakana Phonetic Extensions
+  // Enclosed CJK Letters and Months
+  // CJK Compatibility
+  // CJK Unified Ideographs Extension A
+  // Yijing Hexagram Symbols
+  // CJK Unified Ideographs
+  if (chr >= 0x3100 && chr <=0x9fff) return true;
+  // Hangul Syllables  	
+  if (chr >= 0xac00 && chr <=0xd7af) return true;
+  // Japanes fullwidth alphabet
+  if (chr >= 0xff21 && chr <=0xff5a) return true;
+  // Halfwidth and Fullwidth Forms
+  if (chr >= 0xff00 && chr <=0xffef) return true;
+  // CJK Compatibility Ideographs
+  if (chr >= 0xf900 && chr <=0xfaff) return true;
+  // CJK Unified Ideographs Extension B
+  if (chr >= 0x20000 && chr <=0x2A6DF) return true;
+  // CJK Compatibility Ideographs Supplement  	
+  if (chr >= 0x2F800 && chr <=0x2FA1F) return true;
+  return false; 
+}
+
+
 /**
  * Check if this is a transparent character.
  */
@@ -710,7 +771,9 @@ SGlyph::isTransparent() const
 bool
 SGlyph::isEOP() const
 {
-  return (shared->lineend && shared->precomposed != SD_CD_LS);
+  return (shared->lineend 
+    && shared->precomposed != SD_CD_LS
+    && shared->precomposed != SD_CD_FF);
 }
 
 /**
@@ -719,7 +782,13 @@ SGlyph::isEOP() const
 bool
 SGlyph::isEOL() const
 {
-  return (shared->precomposed == SD_CD_LS);
+  return (shared->precomposed == SD_CD_LS || shared->precomposed == SD_CD_FF);
+}
+
+bool
+SGlyph::isFF() const
+{
+  return (shared->precomposed == SD_CD_FF);
 }
 
 bool
