@@ -16,7 +16,10 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
- 
+
+/*
+ For windows debuging #define UNICODE
+*/
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <fcntl.h>
@@ -37,6 +40,7 @@
 
 #include "SIO.h"
 #include "SExcept.h"
+#include "SEncoder.h"
 
 #ifdef USE_WINAPI
 #define S_MYOFLAGS O_BINARY
@@ -103,7 +107,12 @@ typedef HANDLE SS_DIR;
 typedef DIR* SS_DIR;
 #define SS_INVALID_DIR 0
 #endif
+
+#ifdef USE_WINAPI
+static WCHAR   _currDirW [SS_MAX_DIR_LENGTH];
+#else
 static char    _currDir [SS_MAX_DIR_LENGTH];
+#endif
 
 class SFileImageBuffer
 {
@@ -513,19 +522,37 @@ SFile::getInputStream()
   if (fd<0)
   {
     SString filename=name;
+
 #ifdef USE_WINAPI
     filename.replaceAll("/", "\\");
+    filename.append ((char) 0);
+    SEncoder u8("utf-8");
+    SEncoder u16("utf-16-le");
+    SString res = u16.encode (u8.decode (filename));
+    WCHAR* filenameW = (WCHAR *) res.array();
+#else
+    filename.append ((char) 0);
+    char * filenameU = (char*) filename.array();
 #endif
-    char * buff = filename.cString();
     if (filename.size()==0)
     {
       fd = (long) STDIN;
     }
     else
     {
-      fd = (long) open (buff, O_RDONLY | S_MYOFLAGS );
+#ifdef USE_WINAPI
+
+      fd = (long) _wopen (filenameW, O_RDONLY | S_MYOFLAGS);
+      // Windows98SE does not implement _wopen
+      if (fd < 0) {
+         SString filenameA = utf8ToSystem (filename);
+         filenameA.append ((char)0);
+         fd = (long) open (filenameA.array(), O_RDONLY | S_MYOFLAGS);
+      }
+#else
+      fd = (long) open (filenameU, O_RDONLY | S_MYOFLAGS );
+#endif
     }
-    delete buff;
   }
   SInputStream ins(SEventSource::FILE, fd);
 #ifdef USE_WINAPI
@@ -550,17 +577,33 @@ SFile::getOutputStream()
     SString filename=name;
 #ifdef USE_WINAPI
     filename.replaceAll("/", "\\");
+    filename.append ((char) 0);
+    SEncoder u8("utf-8");
+    SEncoder u16("utf-16-le");
+    SString res = u16.encode (u8.decode (filename));
+    WCHAR* filenameW = (WCHAR *) res.array();
+#else
+    filename.append ((char) 0);
+    char * filenameU = (char*) filename.array();
 #endif
-    char * buff = filename.cString();
-    if (filename.size()==0)
+    if (name.size()==0)
     {
       fd = STDOUT;
     }
     else
     {
-      fd = open (buff, O_WRONLY | O_CREAT | O_TRUNC | S_MYOFLAGS, 0666);
+#ifdef USE_WINAPI
+      fd = _wopen (filenameW, O_WRONLY | O_CREAT | O_TRUNC | S_MYOFLAGS, 0666);
+      // Windows98SE does not implement _wopen
+      if (fd < 0) {
+         SString filenameA = utf8ToSystem (filename);
+         filenameA.append ((char)0);
+         fd = open (filenameA.array(), O_WRONLY | O_CREAT | O_TRUNC | S_MYOFLAGS, 0666);
+      }
+#else
+      fd = open (filenameU, O_WRONLY | O_CREAT | O_TRUNC | S_MYOFLAGS, 0666);
+#endif
     }
-    delete buff;
     SOutputStream outs (SEventSource::FILE, (long)fd);
 #ifdef USE_WINAPI
     outs.setOK(fd>0);
@@ -620,18 +663,36 @@ SFile::getFileImage(bool writeflag)
 #ifdef USE_WINAPI
   fn.replaceAll("/", "\\");
 #endif
-  char* fileName = fn.cString();
+  fn.append ((char) 0);
 
 #ifdef USE_WINAPI
-  //void* fd=CreateFile (fileName, 0, OF_READ | OF_SHARE_DENY_NONE);
-  void* fd=CreateFile (fileName, 
+  //SString p = ("\\\\?\\");
+  //p.append (utf8ToSystem (fn));
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString res = u16.encode (u8.decode (fn));
+  WCHAR* fileNameW = (WCHAR *) res.array();
+#else
+  const char* fileName = fn.array();
+#endif
+
+
+#ifdef USE_WINAPI
+  //void* fd=CreateFileW (fileNameW, 0, OF_READ | OF_SHARE_DENY_NONE);
+  void* fd=CreateFileW (fileNameW, 
 	(writeflag) ? (GENERIC_READ| GENERIC_WRITE) : GENERIC_READ, 
 	(writeflag) ? (FILE_SHARE_READ | FILE_SHARE_WRITE) : FILE_SHARE_READ,
 	0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, 0);
   if (fd==0)
   {
-    delete fileName;
-    return map;
+    // Windows98SE does not implement CreateFileW
+    SString fileNameA = utf8ToSystem (fn);
+    fileNameA.append ((char)0);
+    fd=CreateFileA (fileNameA.array(), 
+ 	(writeflag) ? (GENERIC_READ| GENERIC_WRITE) : GENERIC_READ, 
+	(writeflag) ? (FILE_SHARE_READ | FILE_SHARE_WRITE) : FILE_SHARE_READ,
+	0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL | FILE_FLAG_RANDOM_ACCESS, 0);
+    if (fd == 0) return map;
   }
   if (fd==0)
 #else
@@ -639,16 +700,16 @@ SFile::getFileImage(bool writeflag)
   if (fd<0)
 #endif
   {
-    delete fileName;
     return map;
   }
   SFileImage newMap((long) fd, s, writeflag);
   map = newMap;
   if (map.size() < 0)
   {
+#ifndef USE_WINAPI
     fprintf (stderr, "SIO: Open succeeded but mmap failed for %s\n", fileName);
+#endif
   }
-  delete fileName;
   return map;
 }
 
@@ -669,17 +730,42 @@ SFile::size()
 {
   struct stat     buf;
   SString newName = name;
+  newName.append ((char)0);
 #ifdef USE_WINAPI
   newName.replaceAll("/", "\\");
-#endif
-  char* cname = newName.cString();
-  if (stat (cname, &buf) != 0)
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString u16str = u16.encode (u8.decode (newName));
+  WCHAR* currentW = (WCHAR *) u16str.array();
+ 
+  WIN32_FIND_DATAW entry;
+  SS_DIR  dir=FindFirstFileW (currentW, &entry);
+  if (dir == SS_INVALID_DIR)
+  {
+    WIN32_FIND_DATAA entryA;
+    SString system = utf8ToSystem (newName);
+    system.append ((char)0);
+    // Fall back to system
+    char* currentA = (char *) system.array();
+
+    // Windows98SE does not implement FindFirstFileW
+    dir=FindFirstFileA (currentA, &entryA);
+    if (dir == SS_INVALID_DIR)
+    {
+      return -1;
+    }
+    FindClose (dir);
+    return ((long)entryA.nFileSizeHigh * (MAXDWORD+1)) + (long)entryA.nFileSizeLow;
+  }
+  FindClose (dir);
+  return ((long)entry.nFileSizeHigh * (MAXDWORD+1)) + (long)entry.nFileSizeLow;
+#else
+  if (stat ((char*)newName.array(), &buf) != 0)
   {
-    delete cname;
     return -1;
   }
-  delete cname;
   return (long) buf.st_size;
+#endif
 }
 
 bool
@@ -688,31 +774,45 @@ SFile::truncate (long _size)
   SString newName = name;
 #ifdef USE_WINAPI
   newName.replaceAll("/", "\\");
+  newName.replaceAll("/", "\\");
+  newName.append ((char) 0);
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString res = u16.encode (u8.decode (newName));
+  WCHAR* filenameW = (WCHAR *) res.array();
+#endif
+  SString newNameU = newName;
+  newNameU.append ((char) 0);
+  char * filenameU = (char*) newName.array();
+#ifdef USE_WINAPI
+  int d = _wopen (filenameW, O_WRONLY | O_CREAT | O_APPEND | S_MYOFLAGS, 0666);
+  // Windows98SE does not implement _wopen
+  if (d < 0) {
+     SString filenameA = utf8ToSystem (newName);
+     filenameA.append ((char)0);
+      d = open (filenameA.array(), O_WRONLY | O_CREAT | O_APPEND | S_MYOFLAGS, 0666);
+  }
+#else
+  int d = open (filenameU, O_WRONLY | O_CREAT | O_APPEND | S_MYOFLAGS, 0666);
 #endif
-  char* cname = newName.cString();
-  int d = open (cname, O_WRONLY | O_CREAT | O_APPEND | S_MYOFLAGS, 0666);
   if (d <=0)
   {
-    delete cname;
     return false;
   }
 #ifdef USE_WINAPI
   if (::_chsize (d, _size) != 0)
   {
     close (d);
-    delete cname;
     return false;
   }
 #else
   if (::ftruncate (d, _size) != 0)
   {
     close (d);
-    delete cname;
     return false;
   }
 #endif
   close (d);
-  delete cname;
   return true;
 }
 
@@ -812,7 +912,7 @@ SSocket::openSocket(SOutputStream* o)
 #ifdef USE_WINAPI 
   initsockets (true);
 #endif
-  int sd = sd = socket (AF_INET, SOCK_STREAM, IPPROTO_IP);
+  int sd = socket (AF_INET, SOCK_STREAM, IPPROTO_IP);
   SOutputStream s (SEventSource::SOCKET, (long)sd);
   *o = s;
 #ifdef USE_WINAPI
@@ -900,13 +1000,25 @@ SServer::~SServer ()
 SDir::SDir(void) : name("/home/gsinai")
 {
 #ifdef USE_WINAPI
-  if (GetCurrentDirectory (SS_MAX_DIR_LENGTH-1, _currDir))
+/*
   //char* buff=0;
-  //if (GetFullPathName ("Gazsi Bacsi", SS_MAX_DIR_LENGTH-1, _currDir, &buff))
+  //if (GetFullPathNameQ ("Gazsi Bacsi", SS_MAX_DIR_LENGTH-1, _currDirW, &buff))
+*/
+  if (GetCurrentDirectoryW (SS_MAX_DIR_LENGTH-1, _currDirW))
   {
-    name = SString (_currDir);
+    
+    SString u16fn ((const char*) _currDirW, (unsigned int) 2 * wcslen (_currDirW));
+    SEncoder u8("utf-8");
+    SEncoder u16("utf-16-le");
+
+    name = u8.encode (u16.decode (u16fn));
+
+  // Windows98 does not implement getCurrentDirectoryW
+  } else if (GetCurrentDirectoryA (SS_MAX_DIR_LENGTH-1, (char*) _currDirW)) {
+    SString ansi ((const char*) _currDirW);
+    name = systemToUtf8 (ansi);
   }
-  else
+  else 
   {
     name = (SString) "Z:/";
   }
@@ -930,8 +1042,8 @@ SDir::SDir(void) : name("/home/gsinai")
 SDir::SDir(const SString& _name) : name (_name)
 {
   /* convert it to a real name */
-  name.replace ("\\", "/");
-  name.replace ("//", "/");
+  name.replaceAll ("\\", "/");
+  name.replaceAll ("//", "/");
   if (name.size() == 0 || name[0] != '/')
   {
     name.insert (0, "/");
@@ -1038,76 +1150,124 @@ SDir::list (const SStringVector &patterns, SEntry e)
     return SStringVector (v);
 
   }
-  _name.append("/*.*");
-  _name.replaceAll("//", "/");
-  _name.replaceAll("/", "\\");
+   _name.append("/*.*");
+   _name.replaceAll("//", "/");
+   _name.replaceAll("/", "\\");
+   _name.append ((char)0);
+
+   SEncoder u8("utf-8");
+   SEncoder u16("utf-16-le");
+   SString u16str = u16.encode (u8.decode (_name));
+   WCHAR* currentW = (WCHAR *) u16str.array();
+
+#else 
+   _name.append ((char)0);
+   char* currentU = (char*) _name.array();
 #endif
-  char* current = _name.cString();
-  CHECK_NEW (current);
-  SS_DIR  dir;
+
+   SS_DIR  dir;
+ 
+
 #ifdef USE_WINAPI
-  bool first=true;
-  WIN32_FIND_DATA entry;
-  dir=FindFirstFile (current, &entry);
+  WIN32_FIND_DATAW entry;
+  WIN32_FIND_DATAA entryA;
+  dir=FindFirstFileW (currentW, &entry);
+  boolean ansi = false;
+  if (dir == SS_INVALID_DIR)
+  {
+    SString system = utf8ToSystem (_name);
+    system.append ((char)0);
+    // Fall back to system
+    char* currentA = (char *) system.array();
+
+    // Windows98SE does not implement FindFirstFileW
+    dir=FindFirstFileA (currentA, &entryA);
+    ansi = true;
+  }
 #else
   struct dirent*  entry;
-  dir = opendir (current);
+  dir = opendir (currentU);
 #endif
   if (dir == SS_INVALID_DIR)
   {
-    //fprintf (stderr, "can not opendir: %s\n", current);
-    delete current;
+    //fprintf (stderr, "can not opendir: %s\n", currentU);
     return SStringVector(entries);
   }
   struct stat buf;
   while (true)
   {
+
 #ifdef USE_WINAPI
-    if (first)
-    {
-      first = false;
+
+    SString dnameStr;
+
+    if (ansi) {
+       dnameStr = systemToUtf8 (SString (entryA.cFileName));
+       entry.dwFileAttributes = entryA.dwFileAttributes;
+    } else {
+       SString u16fn ((const char*) entry.cFileName, (unsigned int) 2 * wcslen (entry.cFileName));
+       dnameStr = u8.encode (u16.decode (u16fn));
     }
-    else
+
+    if (entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
     {
-      if (!FindNextFile (dir, &entry))
+      if (e==SE_DIR && dnameStr != ".." && dnameStr != ".")
+      {
+        for(unsigned int i=0; i<patterns.size(); i++)
+        {
+          if (dnameStr.match(patterns[i]))
+          {
+            entries.append (dnameStr); break;
+          }
+        }
+      }
+    }
+    else if (!(entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY))
+    {
+      if (e==SE_FILE)
+      {
+        for(unsigned int i=0; i<patterns.size(); i++)
+        {
+          if (dnameStr.match(patterns[i]))
+          {
+            entries.append (dnameStr); break;
+          }
+        }
+      }
+    }
+    if (ansi) {
+      if (!FindNextFileA (dir, &entryA))
       {
         if (GetLastError()==ERROR_NO_MORE_FILES) break;
         fprintf (stderr, "some errrors...%d\n", (int) GetLastError());
         break;
       }
-    }
+    } else {
+      if (!FindNextFileW (dir, &entry))
+      {
+        if (GetLastError()==ERROR_NO_MORE_FILES) break;
+        fprintf (stderr, "some errrors...%d\n", (int) GetLastError());
+        break;
+      }
+   }
+
 #else
+
     if ((entry=readdir (dir))==0) break;
-#endif
 
     SString fullName = name;
     fullName.append ('/');
-#ifdef USE_WINAPI
-/* SGC */
-    if (fullName.size() > 0 && fullName[0] == '/')
-    {
-      fullName.remove (0); 
-    }
-    char* dname = entry.cFileName;
-#else
     char* dname = entry->d_name;
-#endif
-
     fullName.append (dname);
     fullName.replaceAll("//", "/");
-#ifdef USE_WINAPI
-    fullName.replaceAll("/", "\\");
-#endif
     char* cFullName = fullName.cString();
     CHECK_NEW (cFullName);
-    if (stat (cFullName, &buf) != 0 
-#ifndef USE_WINAPI
-       && lstat (cFullName, &buf) != 0
-#endif
-    )
+
+    if (stat (cFullName, &buf) != 0 && lstat (cFullName, &buf) != 0)
     {
       fprintf (stderr, "can not stat: %s\n", cFullName);
-      delete cFullName; continue;
+      delete cFullName; 
+      continue;
     }
 
     if (SS_S_ISDIR( buf.st_mode))
@@ -1125,12 +1285,7 @@ SDir::list (const SStringVector &patterns, SEntry e)
         }
       }
     }
-    else if (
-         SS_S_ISREG (buf.st_mode)
-#ifndef USE_WINAPI
-      || SS_S_ISLNK (buf.st_mode)
-#endif
-       )
+    else if ( SS_S_ISREG (buf.st_mode) || SS_S_ISLNK (buf.st_mode))
     {
       if (e==SE_FILE)
       {
@@ -1145,6 +1300,8 @@ SDir::list (const SStringVector &patterns, SEntry e)
       }
     }
     delete cFullName;
+#endif
+
   }
 
 #ifdef USE_WINAPI
@@ -1152,7 +1309,6 @@ SDir::list (const SStringVector &patterns, SEntry e)
 #else
   closedir (dir);
 #endif
-  delete current;
   return SStringVector(entries);
 }
 
@@ -1211,7 +1367,34 @@ SDir::exists() const
   {
   }
   _name.replaceAll("/", "\\");
-#endif
+  _name.append ((char)0);
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString u16str = u16.encode (u8.decode (_name));
+  WCHAR* currentW = (WCHAR *) u16str.array();
+ 
+  WIN32_FIND_DATAW entry;
+  SS_DIR  dir=FindFirstFileW (currentW, &entry);
+  if (dir == SS_INVALID_DIR)
+  {
+    WIN32_FIND_DATAA entryA;
+    SString system = utf8ToSystem (_name);
+    system.append ((char)0);
+    // Fall back to system
+    char* currentA = (char *) system.array();
+
+    // Windows98SE does not implement FindFirstFileW
+    dir=FindFirstFileA (currentA, &entryA);
+    if (dir == SS_INVALID_DIR)
+    {
+      return false;
+    }
+    if (entryA.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) return true;
+    return false;
+  }
+  if (entry.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) return true;
+  return false;
+#else
   struct stat buf;
   char* current = _name.cString();
   CHECK_NEW (current);
@@ -1227,6 +1410,7 @@ SDir::exists() const
   }
   delete current;
   return true;
+#endif
 }
 
 /**
@@ -1237,6 +1421,7 @@ bool
 SDir::create () const
 {
   SString s = name;
+
 #ifdef USE_WINAPI
   if (s.size() > 0 && s[0] == '/')
   {
@@ -1244,8 +1429,21 @@ SDir::create () const
   }
   s.replaceAll("/", "\\");
   s.append ((char) 0);
-  int status = mkdir (s.array());
-  if (status == -1) return false;
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString u16str = u16.encode (u8.decode (s));
+
+  WIN32_FIND_DATAW entry;
+  int status= _wmkdir ((WCHAR*) u16str.array());
+  if (status != 0) {
+     // Windows98SE does not implement _wmkdir
+     SString ansi = utf8ToSystem (s);
+     ansi.append ((char)0);
+     status= mkdir (ansi.array ());
+     if (status != 0) {
+       return false;
+     }
+  }
 #else
   s.append ((char) 0);
   int status = mkdir (s.array(), 0755);
@@ -1284,27 +1482,51 @@ SDir::readable() const
   {
   }
   _name.replaceAll("/", "\\");
-#endif
+
+#else
+
   char* current = _name.cString();
   CHECK_NEW (current);
+
+#endif
+
   SS_DIR dir;
+
 #ifdef USE_WINAPI
-  WIN32_FIND_DATA entry;
-  dir=FindFirstFile (current, &entry);
+
+  WIN32_FIND_DATAW entry;
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  _name.append ((char)0);
+  SString u16str = u16.encode (u8.decode (_name));
+  dir=FindFirstFileW ((WCHAR*) u16str.array(), &entry);
+  if (dir==SS_INVALID_DIR)
+  {
+    WIN32_FIND_DATAA entryA;
+    SString system = utf8ToSystem (_name);
+    // Fall back to system
+    char* currentA = (char *) system.array();
+
+    // Windows98SE does not implement FindFirstFileW
+    dir=FindFirstFileA (currentA, &entryA);
+    if (dir==SS_INVALID_DIR) return false;
+  }
 #else
   dir = opendir (current);
-#endif
   if (dir==SS_INVALID_DIR)
   {
     delete current;
     return false;
   }
+#endif
+
 #ifdef USE_WINAPI
   FindClose (dir);
 #else
   closedir(dir);
-#endif
   delete current;
+#endif
+
   return true;
 }
 
@@ -1453,10 +1675,16 @@ SPipe::openPipe(bool output)
      fprintf (stderr, "openPipe failed.\n");
      return -1;
    }
-   STARTUPINFO sinfo;
+   STARTUPINFOW sinfo;
    memset (&sinfo, 0, sizeof (sinfo));
    sinfo.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
    sinfo.wShowWindow = SW_SHOWDEFAULT;
+
+   STARTUPINFOA sinfoA;
+   memset (&sinfoA, 0, sizeof (sinfoA));
+   sinfoA.dwFlags = STARTF_USESHOWWINDOW | STARTF_USESTDHANDLES;
+   sinfoA.wShowWindow = SW_SHOWDEFAULT;
+
    HANDLE hProcess = GetCurrentProcess();
    HANDLE dupedHandle = INVALID_HANDLE_VALUE; 
    HANDLE myHandle = INVALID_HANDLE_VALUE; 
@@ -1470,6 +1698,10 @@ SPipe::openPipe(bool output)
      sinfo.hStdInput = dupedHandle;
      sinfo.hStdOutput = INVALID_HANDLE_VALUE;
      sinfo.hStdError = INVALID_HANDLE_VALUE;
+
+     sinfoA.hStdInput = dupedHandle;
+     sinfoA.hStdOutput = INVALID_HANDLE_VALUE;
+     sinfoA.hStdError = INVALID_HANDLE_VALUE;
    }
    else
    {
@@ -1480,17 +1712,25 @@ SPipe::openPipe(bool output)
      sinfo.hStdInput = INVALID_HANDLE_VALUE;
      sinfo.hStdOutput = dupedHandle;
      sinfo.hStdError = INVALID_HANDLE_VALUE;
+
+     sinfoA.hStdInput = INVALID_HANDLE_VALUE;
+     sinfoA.hStdOutput = dupedHandle;
+     sinfoA.hStdError = INVALID_HANDLE_VALUE;
    }
 
    PROCESS_INFORMATION* pinfo = new PROCESS_INFORMATION[1];
 
-   SString commandline (command);
-   char* cl = commandline.cString();
-  
-   /* create process with inherit handle */ 
-   if (!CreateProcess (
+   SEncoder u8("utf-8");
+   SEncoder u16("utf-16-le");
+   SString cutf8 = command;
+   cutf8.append ((char)0);
+   SString cmdlineString = u16.encode (u8.decode (cutf8));
+   SString cmdlineStringA = utf8ToSystem(cutf8);
+   WCHAR* clW = (WCHAR *) cmdlineString.cString();
+
+   if (!CreateProcessW (
         NULL, // module name 
-        cl, // command line 
+        clW, // command line 
         NULL, // not inheritable handles
         NULL, // not inheritable thread handle
         TRUE, // handles not inherited 
@@ -1498,16 +1738,28 @@ SPipe::openPipe(bool output)
         NULL, // use parent's environment
         NULL, // use parent's starting directory
         &sinfo, // STARTUP_INFO
+        pinfo) // PROCESS_INFO
+     // Windows98SE does not implement CreateProcessW
+
+    && !CreateProcessA (
+        NULL, // module name 
+        cmdlineStringA.cString(), // command line 
+        NULL, // not inheritable handles
+        NULL, // not inheritable thread handle
+        TRUE, // handles not inherited 
+        CREATE_DEFAULT_ERROR_MODE | DETACHED_PROCESS, // no console
+        NULL, // use parent's environment
+        NULL, // use parent's starting directory
+        &sinfoA, // STARTUP_INFO
         pinfo)) // PROCESS_INFO
    {
-     fprintf (stderr, "Could not create process [%s]\n", cl);
+
+     fprintf (stderr, "Could not create process [%s]\n", clW);
      CloseHandle (myHandle);
      CloseHandle (dupedHandle);
      delete pinfo;
-     if (cl) delete cl;
      return -1;
    }
-   if (cl) delete cl;
    /* we need to close this because it got duped */
    CloseHandle (dupedHandle);
 
@@ -1517,18 +1769,19 @@ SPipe::openPipe(bool output)
    waitHandles.append ((long)pinfo);
    return (long) myHandle;
 #else
+   SString cmdlineString (command);
    int pp[2];
    if (pipe (pp) == -1)
    {
      fprintf (stderr, "openPipe failed.\n");
      return -1;
    }
-   char* c = command.cString();
+   char* clString = cmdlineString.cString();
    int pid = fork();
    if (pid>0)
    {
+     delete clString;
      waitHandles.append (pid);
-     delete c;
      if (output)
      {
        close (pp[0]);
@@ -1541,7 +1794,6 @@ SPipe::openPipe(bool output)
    }
    else if (pid<0)
    {
-     delete c;
      close (pp[1]);
      close (pp[0]);
      return -1;
@@ -1579,7 +1831,8 @@ SPipe::openPipe(bool output)
 //fprintf (stderr, "return child %d\n", pp[1]);
        close (pp[1]);
      }
-     execl ("/bin/sh", "sh", "-c", c, (char*) 0);
+     execl ("/bin/sh", "sh", "-c", clString, (char*) 0);
+     delete clString;
      exit (127);
    }
 #endif
@@ -1589,24 +1842,42 @@ SPipe::openPipe(bool output)
  * Create a temporary file.
  */
 #ifdef USE_WINAPI
-static char    _tmpFile [SS_MAX_DIR_LENGTH];
+static WCHAR   _tmpFileW [SS_MAX_DIR_LENGTH];
 #endif
 
 SString
 getTemporaryFileName ()
 {
 #ifdef USE_WINAPI
-  if (!GetTempPath (sizeof (_currDir)-1, _currDir))
+  // Windows98SE W does not work.
+  if (!GetTempPathW (SS_MAX_DIR_LENGTH-1, _currDirW))
   {
-    fprintf (stderr, "Can not get temporary directory.\n");
-    return SString ("");
+    if (!GetTempPathA (SS_MAX_DIR_LENGTH-1, (char*) _currDirW))
+    {
+      fprintf (stderr, "Can not get temporary directory.\n");
+      return SString ("");
+    }
+    if (!GetTempFileNameA ((char*)_currDirW, "yudit", 0, (char*) _tmpFileW))
+    {
+      fprintf (stderr, "Can not get temporary filename.\n");
+      return SString ("");
+    }
+    SString ansi ((char*) _tmpFileW); 
+    SString utf8 = systemToUtf8 (ansi);
+    return SString (utf8);
   }
-  if (!GetTempFileName (_currDir, "yudit", 0, _tmpFile))
+  if (!GetTempFileNameW (_currDirW, L"yudit", 0, _tmpFileW))
   {
     fprintf (stderr, "Can not get temporary filename.\n");
     return SString ("");
   }
-  return SString (_tmpFile);
+  SString u16fn ((const char*) _tmpFileW, (unsigned int) 2 * wcslen (_tmpFileW));
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+
+  SString name = u8.encode (u16.decode (u16fn));
+  return SString (name);
+
 #else
   SString ret("/tmp");
   int pid= getpid();
