@@ -1,7 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -997,7 +997,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
   SS_UCS4 composition;
   bool    usePrecomposed;
   bool    isShaped;
-  bool    isLigature;
   bool    isLineEnd;
   //gl->ensure (ucs4.size());
   SGlyphShared* shr = 0;
@@ -1013,7 +1012,6 @@ split (const SV_UCS4& ucs4, SVector<SGlyph>* gl, unsigned int from)
     compIndex = 0;
     usePrecomposed = false;
     isShaped = false;
-    isLigature = false;
     isLineEnd = true;
     unsigned int clusterIndex = 0;
     SV_UCS4 du4;
