@@ -28,6 +28,7 @@
 #include "swindow/SColor.h"
 #include "stoolkit/SGlyph.h"
 #include "stoolkit/SParagraph.h"
+#include "stoolkit/SLineTracker.h"
 
 class STextDataEvent
 {
@@ -47,13 +48,13 @@ public:
   STextIndex    start;
   /* a reversely calcualted value */
   STextIndex    remaining;
-
 };
 
 class STextDataListener
 {
 public:
   virtual void textChanged(void* src, const STextDataEvent& event) = 0;
+  virtual ~STextDataListener() {}
 };
 
 /**
@@ -70,8 +71,13 @@ public:
   SString getText () const;
   SString getText (const STextIndex& index) const;
   SString getText (const STextIndex& begin, const STextIndex& end) const;
+
+  SV_UCS4 getChars (unsigned int line) const
+   { return lines[line]->getChars (); }
   STextIndex find (const SString& string);
   const SGlyph& glyphAt (const STextIndex& index) const;
+  // we need only this
+  const SGlyph* peekGlyphAt (const STextIndex& index) const;
 
   /* these, till fireEvent all work with events */
   void move (const STextIndex& index);
@@ -105,6 +111,9 @@ public:
 
   unsigned int  size() const;
   unsigned int  size(unsigned int line) const;
+  // For syntax, It will not expand line
+  unsigned int  softSize (unsigned int line) const;
+  SS_UCS4       softCharAt (unsigned int line, unsigned int pos) const;
 
   STextIndex getTextIndex (int charOffset=0, bool logical=false) const;
   STextIndex getTextIndex(const STextIndex& base, int charOffset=0, bool logical=false) const;
@@ -114,7 +123,11 @@ public:
   unsigned int toLogical (unsigned int line, unsigned int index);
   SV_UINT getLogicalMap(unsigned int line) const;
 
+  // Set only
   void addTextDataListener (STextDataListener* listener);
+  // Set only - lineExpanded is never called, it mmust be handled separately
+  void addLineTracker (SLineTracker* lt);
+
   bool isProperLine (unsigned int line) const;
   bool isVisible(unsigned int line) const;
   void setVisible(unsigned int line);
@@ -127,7 +140,9 @@ public:
 
   bool isWhiteSpace (const STextIndex& index) const;
   bool isDelimiter (const STextIndex& index) const;
+  bool canWrap (const STextIndex& index) const;
   bool isNumber (const STextIndex& index) const;
+  bool isLetter (const STextIndex& index) const;
   bool isTransparent (const STextIndex& index) const;
 
   void setLineBreaks (unsigned int line, const SV_UCS4& breaks);
@@ -135,6 +150,14 @@ public:
   bool getInitialLR() const;
   void setInitialLR(bool lr);
 
+  const STextDataEvent& getCurrentEvent () const { return  event; }
+
+  bool isExpanded (unsigned int line) const
+     { return lines[line]->isExpanded(); }
+
+  SS_ParaSep getParagraphSeparator (unsigned int line) const
+     { return lines[line]->getParagraphSeparator (); }
+
 private:
   SS_Embedding         embedding;
 
@@ -159,6 +182,7 @@ private:
   /* should be a linked list */
   SBinVector<SParagraph*>  lines;
   STextDataListener*     listener;
+  SLineTracker*          lineTracker;
 };
 
 #endif /* STextData_h */
