@@ -20,7 +20,7 @@
 #ifndef STypes_h
 #define STypes_h
 
-#define SD_YUDIT_VERSION "2.8.1"
+#define SD_YUDIT_VERSION "2.9.6"
 
 // Get the SS_WORD16_t and u_int_32
 #include <sys/types.h>
@@ -74,6 +74,9 @@ typedef int SAlignment;
 typedef SS_WORD16 SS_GlyphIndex;
 typedef SBinVector<SS_GlyphIndex> SV_GlyphIndex;
 
+typedef  SBinVector<int> SSyntaxRow;
+typedef  SBinVector<SSyntaxRow*> SSyntaxData;
+typedef  SBinVector<SV_UCS4*> SUnicodeData;
 
 #define SD_Left ((SAlignment)-1)
 #define SD_Right ((SAlignment)1)
@@ -82,5 +85,13 @@ typedef SBinVector<SS_GlyphIndex> SV_GlyphIndex;
 /* char can be only positive on some platforms */
 #define SD_NOSHAPE ((char)0x7f)
 
+/* This many lines are pre-expanded around cursor, for efficiencey */
+#define SD_PRE_EXPAND 50
+#define DEBUG_PARSER 0
+
+#define SD_MATCH_EOD 0x7ffffff
+// to provide timer based solution
+#define SD_MATCH_AGAIN 0x7fffffe
+
 
 #endif /* STypes_h */
