@@ -34,6 +34,7 @@
 #ifdef USE_WINAPI
 #include <wtypes.h>
 #include <winreg.h> 
+#include <ShlObj.h>
 #else
 #include <unistd.h> 
 #include <pwd.h>
@@ -44,22 +45,41 @@
 
 SString getHome ()
 {
-#ifndef USE_WINAPI
-  struct passwd * p = getpwuid(getuid());
-  if (p!=0)
-  {
-    return SString (p->pw_dir);
-  }
-#endif
-  char* henv = getenv ("HOME");
+#ifdef USE_WINAPI
+  // default
+  SString ret ="C:/HOME";
+  // First try the HOME environment
+  WCHAR* henv = _wgetenv (L"HOME");
   if (henv)
   {
-    SString h (henv);
-    h.replace ("\\", "/");
-    return SString(h);
+    SString u16fn ((const char*) henv, 2 * (unsigned int) wcslen (henv));
+    SEncoder u8("utf-8");
+    SEncoder u16("utf-16-le");
+    SString h = u8.encode (u16.decode (u16fn));
+    h.replaceAll ("\\", "/");
+    SDir dir (h);
+    if (dir.exists()) {
+      return SString (h);
+    }
+    fprintf (stderr, "HOME environment variable is nonexistent directory : \"%*.*s\".\n", SSARGS (h));
   }
-#ifdef USE_WINAPI
-  SString ret ="C:/HOME";
+
+  WCHAR  *full = new WCHAR[MAX_PATH+1];
+  CHECK_NEW (full);
+  full[MAX_PATH] = 0;
+  // Surpsisingly this works on Windows98SE
+  if (SHGetSpecialFolderPathW (0, full, CSIDL_PERSONAL, false)) {
+    SString u16fn ((const char*) full, 2 * (unsigned int) wcslen (full));
+    SEncoder u8("utf-8");
+    SEncoder u16("utf-16-le");
+    ret = u8.encode (u16.decode (u16fn));
+  } else if (SHGetSpecialFolderPathA (0, (char *) full, CSIDL_PERSONAL, false)) {
+    SString s ((char*)full);
+    ret = systemToUtf8 (s);
+  }
+  delete full;
+  ret.replaceAll ("\\", "/");
+
   SDir dir (ret);
   if (dir.exists())
   {
@@ -78,6 +98,27 @@ SString getHome ()
   }
   return ret;
 #else
+  // HOME comes first
+  char* henv = getenv ("HOME");
+  if (henv)
+  {
+    SString h (henv);
+    if (h.size() != 0 && h[0] == '/') {
+        SDir dir (h);
+        if (dir.exists()) {
+            return SString(h);
+        } else {
+           fprintf (stderr, "HOME environment variable is nonexistent directory : \"%*.*s\".\n", SSARGS (h));
+        }
+    } else {
+       fprintf (stderr, "HOME environment variable is not absolute path: \"%*.*s\".\n", SSARGS (h));
+    }
+  }
+  struct passwd * p = getpwuid(getuid());
+  if (p!=0)
+  {
+    return SString (p->pw_dir);
+  }
   fprintf (stderr, "can not determine home directory. Using /tmp");
   return "/tmp";
 #endif
@@ -87,8 +128,9 @@ SString getHome ()
 #define YUDIT_DATA "/usr/share/yudit"
 #endif /* YUDIT_DATA  */
 
+#define YUDIT_MAX_DATA 2048
+
 #ifdef USE_WINAPI
-char _registryBuffer[1024];
 static SString 
 getReg (const SString& _base, const SString& _name)
 {
@@ -96,40 +138,80 @@ getReg (const SString& _base, const SString& _name)
   SString key = _base; 
   SString name = _name;
   key.append ("\\"); key.append (name); key.append ((char)0);
-  DWORD datasize = sizeof (_registryBuffer);
+
+  DWORD datasize = YUDIT_MAX_DATA;
+  char* _registryBuffer = new char[datasize];
+  CHECK_NEW (_registryBuffer);
   DWORD type;
   LONG opened;
   HKEY hkey=0;
   type = REG_SZ;
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString utf16key = u16.encode (u8.decode (key));
   if (
-     (opened = RegOpenKeyEx (
-       HKEY_LOCAL_MACHINE, key.array(), 0, KEY_READ, &hkey)
+
+     (opened = RegOpenKeyExW (
+       HKEY_LOCAL_MACHINE, (WCHAR*) utf16key.array(), 0, KEY_READ, &hkey)
        == ERROR_SUCCESS) && 
-     RegQueryValueEx (
+     RegQueryValueExW (
        hkey, 0, 0, &type,
-       (unsigned char*)_registryBuffer,
-       &datasize) == ERROR_SUCCESS  && datasize > 2
+       (BYTE*) _registryBuffer,
+       &datasize) == ERROR_SUCCESS  && datasize > 2 && datasize != YUDIT_MAX_DATA
        && (type == REG_SZ 
         || type == REG_EXPAND_SZ))
   {
-    SString ret = SString(_registryBuffer,  (unsigned int) datasize-1);
+    if (datasize < 2) {
+      datasize = 2;
+    }
+    SString u16vle ((const char*) _registryBuffer, (unsigned int) (datasize-2));
+    SString ret = u8.encode (u16.decode (u16vle));
     RegCloseKey (hkey);
+    delete _registryBuffer;
     return SString(ret);
   } 
   if (opened == ERROR_SUCCESS)
   {
     RegCloseKey (hkey); hkey=0;
   }
+  // Windows98SE does not implement RegOpneKeyExW and RegQueryValueExW
+  if (
+     (opened = RegOpenKeyExA (
+       HKEY_LOCAL_MACHINE, key.array(), 0, KEY_READ, &hkey)
+       == ERROR_SUCCESS) && 
+     RegQueryValueExA (
+       hkey, 0, 0, &type,
+       (BYTE*) _registryBuffer,
+       &datasize) == ERROR_SUCCESS  && datasize > 2 && datasize != YUDIT_MAX_DATA
+       && (type == REG_SZ 
+        || type == REG_EXPAND_SZ))
+  {
+    if (datasize < 1) {
+      datasize = 1;
+    }
+    SString vle ((const char*) _registryBuffer, (unsigned int) (datasize-1));
+    SString ret = systemToUtf8 (vle);
+    RegCloseKey (hkey);
+    delete _registryBuffer;
+    return SString(ret);
+  }  
+  if (opened == ERROR_SUCCESS)
+  {
+    RegCloseKey (hkey); hkey=0;
+  }
+  delete _registryBuffer;
   return SString();
 }
 
+/* 
+ * we dont have administrator privilage so this myght get virtualized.
+ */
 static bool 
 setReg (const SString& _base, const SString& _name, const SString& _vle)
 {
   SString key = _base; 
   SString name = _name;
   key.append ("\\"); key.append (name); key.append ((char)0);
-  DWORD datasize = sizeof (_registryBuffer);
   DWORD type;
   LONG opened;
   HKEY hkey=0;
@@ -141,15 +223,20 @@ setReg (const SString& _base, const SString& _name, const SString& _vle)
   sa.nLength = sizeof (SECURITY_ATTRIBUTES);
   sa.lpSecurityDescriptor = NULL;
   sa.bInheritHandle = TRUE;
+  SEncoder u8("utf-8");
+  SEncoder u16("utf-16-le");
+  SString utf16vle = u16.encode (u8.decode (vle));
+  SString utf16key = u16.encode (u8.decode (key));
   if (
-     (opened = RegCreateKeyEx (
-       HKEY_LOCAL_MACHINE, key.array(), 0, 
-       "Why is this parameter not documented?", REG_OPTION_NON_VOLATILE,
-       KEY_WRITE | KEY_READ, &sa, &hkey, &dispose)
+     // Windows98SE does not have the W version and it does not fail.
+     (opened = RegCreateKeyExA (
+       HKEY_LOCAL_MACHINE, (char*) key.array(), 0, 
+       0, REG_OPTION_NON_VOLATILE,
+       KEY_WRITE, &sa, &hkey, &dispose)
        == ERROR_SUCCESS) && 
-     RegSetValueEx (
+     RegSetValueExA (
        hkey, 0, 0, type,
-       (unsigned char*)vle.array(),
+       (BYTE*)vle.array(),
        vle.size()) == ERROR_SUCCESS)
   {
     RegCloseKey (hkey);
@@ -685,3 +772,120 @@ commandExists (const SString& str)
   return false;
 }
 
+SString systemToUtf8 (const SString& str) {
+#ifdef USE_WINAPI
+  if (str.size() == 0) {
+    return SString (str);
+  }
+
+  int  blen = 4 * str.size ();
+  WCHAR*  utf16 = new WCHAR[blen];
+  CHECK_NEW (utf16);
+
+  char* cstr = str.cString();
+
+  int rlen = MultiByteToWideChar (CP_ACP, MB_PRECOMPOSED, cstr, str.size(), utf16, blen);
+  if (rlen > 0) {
+    int i;
+    SV_UCS4 uni;
+    for (i=0; i<rlen; i++) {
+      // FIXME UTF16 surrogates 
+      SS_UCS4 u16 = (SS_UCS4) utf16[i];
+      // 0xd800 and 0xdc00
+      if (u16 >= 0xd800 &&  u16 < 0xe000) {
+         if (i+1 == rlen) {
+           uni.append (u16);
+         } else if (u16 < 0xdc00) {
+           // Sequence is 
+           // U+D800-U+DBFF (High Surrogates)
+           // U+DC00-U+DFFF (Low Surrogates) 
+           SS_UCS4 full = (u16 & 0x3ff) << 10;
+           i++;
+           u16 = (SS_UCS4) utf16[i];
+           full += (u16 & 0x3ff);
+           full += 0x10000;
+           uni.append (full);
+         } else {
+           uni.append (u16);
+           i++;
+           u16 = (SS_UCS4) utf16[i];
+           uni.append (u16);
+         }
+      } else {
+        uni.append (u16);
+      }
+    }
+    delete [] utf16;
+    delete [] cstr;
+    SEncoder enc("utf-8");
+    return (enc.encode (uni));
+  } 
+  delete [] utf16;
+  delete [] cstr;
+  return SString (str);
+#else
+  SString ret;
+  for (unsigned int i=0; i<str.size(); i++) {
+     if (str[i] >= 'a' && str[i] <= 'z') {
+        ret.append ((char) (str[i] - 'a' + 'A'));
+     } else {
+        ret.append ((char) (str[i]));
+     }
+  }
+  return SString (ret);
+#endif
+}
+
+SString utf8ToSystem (const SString& str) {
+#ifdef USE_WINAPI
+  if (str.size() == 0) {
+    return SString (str);
+  }
+
+  SEncoder encoder ("utf-8");
+  SV_UCS4 input = encoder.decode (str);
+
+  int  blen = 2 * input.size ();
+  WCHAR*  utf16 = new WCHAR[blen];
+  CHECK_NEW (utf16);
+  int i;
+  int count = 0;
+  for (i=0; i<input.size(); i++) {
+     SS_UCS2 c = (SS_UCS2) input[i];
+     if (c < 0x10000) {
+        utf16[count++] = c;
+     } else {
+        c = (c - 0x10000);
+        WCHAR h = ((c >> 10) & 0x3ff) | 0xd800;
+        WCHAR l = (c & 0x3ff) | 0xdc00;
+        utf16[count++] = h;
+        utf16[count++] = l;
+     }
+  }
+  blen = 16 * count + 1;
+  char*  out = new char[blen];
+  CHECK_NEW (out);
+  out[blen-1] = 0;
+
+  int rlen = WideCharToMultiByte (CP_ACP, 0, utf16, count, out, blen, NULL, NULL);
+  if (rlen > 0) {
+    SString ret (out, rlen);
+    delete [] utf16;
+    delete [] out;
+    return (SString(ret));
+  } 
+  delete [] utf16;
+  delete [] out;
+  return SString (str);
+#else
+  SString ret;
+  for (unsigned int i=0; i<str.size(); i++) {
+     if (str[i] >= 'A' && str[i] <= 'Z') {
+        ret.append ((char) (str[i] - 'A' + 'a'));
+     } else {
+        ret.append ((char) (str[i]));
+     }
+  }
+  return SString (ret);
+#endif
+}
