@@ -60,6 +60,9 @@ bool isWindows (); /* for nt this is false */
 bool isWinAPI (); /* Microsoft tthis is true */
 bool commandExists (const SString& str);
 
+/* Filnames with codepages are converted to utf-8 here. */
+SString systemToUtf8 (const SString& str);
+SString utf8ToSystem (const SString& str);
 
 
 #endif /* Util_h */
