@@ -83,7 +83,7 @@ SB_UTF8::encode (const SV_UCS4& input)
     }
     if (ini >= 0x4000000)
     {
-      sstring.append ((char) (0xfc | (ini >> 30) & 0x3f));
+      sstring.append ((char) (0xfc | ((ini >> 30) & 0x3)));
       sstring.append ((char) (0x80 | ((ini >> 24) & 0x3f)));
       sstring.append ((char) (0x80 | ((ini >> 18) & 0x3f)));
       sstring.append ((char) (0x80 | ((ini >> 12) & 0x3f)));
@@ -93,7 +93,7 @@ SB_UTF8::encode (const SV_UCS4& input)
     }
     if (ini >= 0x200000)
     {
-      sstring.append ((char) (0xf8 | (ini >> 24) & 0x3f));
+      sstring.append ((char) (0xf8 | ((ini >> 24) & 0x7)));
       sstring.append ((char) (0x80 | ((ini >> 18) & 0x3f)));
       sstring.append ((char) (0x80 | ((ini >> 12) & 0x3f)));
       sstring.append ((char) (0x80 | ((ini >> 6) & 0x3f))); 
