@@ -134,7 +134,8 @@ key==SWindowListener::Key_Shift_L)
   SS_Embedding embd =  editorIF->getDocumentEmbedding();
   cleared = false;
   SString lineBreak = paragBreak;
-  if (ctrl||meta) lineBreak = SS_LB_LS;
+  if (meta) lineBreak = SS_LB_LS;
+  if (ctrl) lineBreak = SS_LB_FF;
   
   /* If there's a selection, erase it first before adding new input*/
   if ((key  != SWindowListener::Key_F1) &&
