@@ -120,7 +120,6 @@ SFileDialog::SFileDialog (void)
 bool
 SFileDialog::setFileType (const SString& _fileType)
 {
-  /* add this filetype */
   bool exists = true;
   if (!fileTypeListBox->selectText (_fileType))
   {
@@ -212,6 +211,8 @@ SFileDialog::rereadDir()
   }
   folderListBox->setText(combedDirs);
   folderNameLabel->setText (currentDirectory.getUnixName ());
+  // This bug has been with us for some time now :)
+  fileListBox->selectText (fileTextEdit->getText());
 }
 
 void
@@ -433,7 +434,7 @@ SFileDialog::recalc ()
 
   setMinimumSize (preferredSize);
 
-  resize (preferredSize);
+  resize (SDimension (preferredSize.width, preferredSize.height));
 
 }
 
@@ -474,6 +475,7 @@ SFileDialog::getInput (const SString& titleString,
   fileFilter = "*";
   filterTextEdit->setText (fileFilter);
   rereadDir ();
+  fileListBox->selectText (fileTextEdit->getText());
   center();
   isCancel = true;
   isFocused = false;
@@ -499,10 +501,10 @@ SFileDialog::buttonPressed (void* source, const SAccelerator* acc)
     bool hasSlash = false;
     for (ck=0; ck<s.size(); ck++)
     {
-       if (s[ck] <= ' ') break;
+       if ((int) (unsigned char) s[ck] <= ' ') break;
        if (s[ck] == '/') hasSlash = true;
     }
-    if (ck==0 || ck<s.size())
+    if (ck==0 || ck+1 == s.size())
     {
       textDialog->getInput ("Error", 
            translate ("Will not create\nspecified folder\n"),
@@ -607,7 +609,9 @@ SFileDialog::itemSelected (void* source, const SAccelerator* acc)
   if (source == fileListBox->textList)
   {
      SString s = fileListBox->textList->getLastSelectedText();
-     fileTextEdit->setText (s);
+     if (s != fileTextEdit->getText()) {
+         fileTextEdit->setText (s);
+     }
      return;
   }
   if (source == fileTypeListBox->textList)
@@ -636,6 +640,16 @@ SFileDialog::textEntered (void *source)
     return;
   }
 }
+
+void
+SFileDialog::textChanged (void *source)
+{
+  if (source == fileTextEdit)
+  {
+     fileListBox->selectText (fileTextEdit->getText());
+  }
+}
+
 void
 SFileDialog::filterChanged ()
 {
