@@ -1,10 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 2003  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -24,6 +21,7 @@
 #define SFrame_h
 
 #include "swidget/SPanel.h"
+#include "swindow/SImage.h"
 
 class SFrameListener
 {
@@ -43,6 +41,7 @@ public:
   virtual void redraw(SWindow* w, int x, int y,
      unsigned int width ,unsigned int height);
   virtual bool windowClose (SWindow* w);
+  virtual void setApplicationImage (const SImage& image);
 protected:
   SFrameListener* frameListener;
 };
