@@ -549,8 +549,16 @@ STextEdit::scrollVertical (int value, bool notify)
   SLocation v = textView.getViewPort();
   v.y = v.y + value;
   textView.setViewPort (v);
+  /* redraw whole thing */
+  if (window->isDoubleBufferEnabled ())
+  {
+    window->redraw (true,
+       (int) border.getBorderSize().width ,
+       (int) border.getBorderSize().height,
+       ewidth, eheight);
+  }
   /* outof range. */
-  if ((value < 0 && -value >= eheight) || (value > 0 && value >= eheight))
+  else if ((value < 0 && -value >= eheight) || (value > 0 && value >= eheight))
   {
     window->redraw (true,
        (int) border.getBorderSize().width ,
@@ -609,8 +617,16 @@ STextEdit::scrollHorizontal (int value, bool notify)
   int ewidth = (int)size.width - (int)2 * border.getBorderSize().width 
        + 2 * SS_LEFT_MARGIN;
   if (ewidth <= 0) return;
+  /* redraw whole thing */
+  if (window->isDoubleBufferEnabled ())
+  {
+    window->redraw (true,
+       (int) border.getBorderSize().width ,
+       (int) border.getBorderSize().height,
+       ewidth, eheight);
+  }
   /* outof range. */
-  if ((value < 0 && -value >= ewidth) || (value > 0 && value >= ewidth))
+  else if ((value < 0 && -value >= ewidth) || (value > 0 && value >= ewidth))
   {
     window->redraw (true,
        (int) border.getBorderSize().width ,
@@ -1005,6 +1021,7 @@ STextEdit::setInputMethod (const SString& _im)
     if (window->getInputMethod()=="x-utf-8" 
      || window->getInputMethod() == "x-ascii" 
      || window->getInputMethod() == ""
+     || window->hasStatusArea() == false
      || ximProperties.get ("InputStyle")  == 0
      || (ximProperties["InputStyle"] != "preedit-under-status-under"
          && ximProperties["InputStyle"] != "preedit-over-status-under")) 
@@ -1050,7 +1067,8 @@ STextEdit::insertClipboardText ()
   for (unsigned int i=0; i<s4.size(); i++)
   {
     SS_UCS4 u = s4[i];
-    if (u==SD_CD_CR || u==SD_CD_LF || u==SD_CD_LS || u==SD_CD_PS) break;
+    if (u==SD_CD_CR || u==SD_CD_LF || u==SD_CD_LS 
+       || u==SD_CD_PS || u==SD_CD_FF) break;
     vle.append ((SS_UCS4) u);
   }
   textView.textData.insert (oc.encode(vle));
@@ -1080,6 +1098,19 @@ STextEdit::clipSelection ()
   window->putClipUTF8 (str);
 }
 
+// Return an empty string if there is no selected text, or
+// return the text currently selected.
+SString
+STextEdit::getSelectedText ()
+{
+  if (startSelection == endSelection)
+  {
+    return SString("");
+  }
+  SString str = textView.textData.getText (startSelection, endSelection);
+  return SString (str);
+}
+
 /**
  * Dirty text may conta8ing newline characters and the line. 
  * This routine is calling insertText for multiline widget.
@@ -1109,17 +1140,23 @@ STextEdit::insertDirtyText (const SString& str, bool embed)
   int ind0 = str.find (SS_LB_DOS); 
   int ind1 = str.find (SS_LB_MAC); 
   int ind2 = str.find (SS_LB_UNIX); 
-  int ind3 = str.find (SS_LB_LS); 
+  int ind3 = str.find (SS_LB_PS); 
+  int ind4 = str.find (SS_LB_LS); 
+  int ind5 = str.find (SS_LB_FF); 
 
   if (ind0 < 0) ind0 = str.size();
   if (ind1 < 0) ind1 = str.size();
   if (ind2 < 0) ind2 = str.size();
   if (ind3 < 0) ind3 = str.size();
+  if (ind4 < 0) ind4 = str.size();
+  if (ind5 < 0) ind5 = str.size();
 
   int smallest = ind0;
   if (ind1 < smallest) smallest = ind1;
   if (ind2 < smallest) smallest = ind2;
   if (ind3 < smallest) smallest = ind3;
+  if (ind4 < smallest) smallest = ind4;
+  if (ind5 < smallest) smallest = ind5;
 
   if (smallest < (int)str.size())
   {
@@ -1180,9 +1217,9 @@ STextEdit::insertText (const SString& _str, bool embed)
     }
   }
   textView.textData.insert (str);
-  textView.textData.fireEvent();
-  /* move to the logical end of previous character */
+  textView.textData.fireEvent ();
 
+  /* move to the logical end of previous character */
   updateCaretPosition ();
   updateCaretLocation ();
 
@@ -1196,7 +1233,7 @@ STextEdit::insertText (const SString& _str, bool embed)
 void
 STextEdit::pageUp ()
 {
-  if (textView.lineHeight == 0 && !isMultiline() 
+  if ((textView.lineHeight == 0 && !isMultiline()) 
      || textView.textData.size ()==0)
   {
     return;
@@ -1262,7 +1299,7 @@ STextEdit::pageUp ()
 void
 STextEdit::pageDown ()
 {
-  if (textView.lineHeight == 0 && !isMultiline() 
+  if ((textView.lineHeight == 0 && !isMultiline()) 
      || textView.textData.size ()==0)
   {
     return;
@@ -1734,7 +1771,8 @@ STextEdit::backspace ()
   if (indexPrev == indexNow) return SString("");
   SString ret = textView.textData.getText (indexPrev);
   textView.textData.remove (indexPrev);
-  textView.textData.fireEvent();
+  // fireTextDataEvent is used now. textView.textData.fireEvent();
+  textView.textData.fireEvent ();
   fireTextChanged ();
   updateCaretPosition ();
   updateCaretLocation ();
@@ -2372,16 +2410,28 @@ STextEdit::getCaretArrowFrom ()
 }
 
 void
-STextEdit::setHighlightMode (STextView::SS_HighlightMode hlMode)
+STextEdit::setSyntax (const SString& hlMode)
 {
-  textView.setHighlightMode(hlMode);
+  textView.setSyntax(hlMode);
   redraw ();
 }
 
-STextView::SS_HighlightMode
-STextEdit::getHighlightMode () const
+const SString&
+STextEdit::getSyntax () const
 {
-  return textView.getHighlightMode();
+  return textView.getSyntax();
+}
+
+void
+STextEdit:: setSyntaxColors (const SSyntaxColors& attr)
+{
+  textView.setSyntaxColors(attr);
+}
+
+const SSyntaxColors&
+STextEdit::getSyntaxColors () const
+{
+  return textView.getSyntaxColors();
 }
 
 void
@@ -2550,8 +2600,8 @@ STextEdit::changeDirection (SS_DR_Dir dir)
     deselectText();
   }
   /* just reset */
-  if (skip || (dir == SS_DR_L || dir == SS_DR_R) 
-     && startSelection == endSelection)
+  if (skip || ((dir == SS_DR_L || dir == SS_DR_R) 
+     && startSelection == endSelection))
   {
     /* pop embedding direction and return 1 if 
      * next direction is different. Also lower the 
