@@ -255,7 +255,11 @@ STextList::valueChanged (SSlidable* _slidable, SSlideType type)
   unsigned int udiff = (diff<0) ? (unsigned int) (-diff) : (unsigned int) diff;
   slidable.value = _slidable->value;
   /* redraw the whole thing */
-  if (udiff > viewableSize.height)
+  if (window->isDoubleBufferEnabled())
+  {
+    window->redraw (true, 0, 0, window->getWidth(), window->getHeight());
+  }
+  else if (udiff > viewableSize.height)
   {
     window->redraw (true,
       (int) border.getBorderSize().width ,
