@@ -18,6 +18,7 @@
  */
 
 #include "swidget/STextView.h"
+#include "stoolkit/SCluster.h"
 
 #define DEBUG_SPEED 0
 
@@ -70,7 +71,8 @@ STextView::STextView (void)
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
-  highlightMode = HM_None;
+  printerPageSize = 0;
+  highlightMode = "";
   isWordWrapOn = false;
   isEditable = false;
   clipx = clipy = 0;
@@ -78,7 +80,10 @@ STextView::STextView (void)
   fontSize = 16.0;
   lineend = true;
   multiline = true;
+  syntax.setTextData (&textData);
+  syntax.addTextDataListener (this);
   textData.addTextDataListener (this);
+  textData.addLineTracker (&syntax);
 }
 
 /**
@@ -90,7 +95,8 @@ STextView::STextView (const SString& utf8)
   rlpen (SColor (0.0, 0.0, 1.0, 1.0)),
   underlineColor("red")
 {
-  highlightMode = HM_None;
+  printerPageSize = 0;
+  highlightMode = "";
   isWordWrapOn = false;
   isEditable = false;
   clipx = clipy = 0;
@@ -99,7 +105,10 @@ STextView::STextView (const SString& utf8)
   lineend = true;
   multiline = true;
   wrapAndPosition ();
+  syntax.setTextData (&textData);
+  syntax.addTextDataListener (this);
   textData.addTextDataListener (this);
+  textData.addLineTracker (&syntax);
   textData.clearEvent();
 }
 
@@ -329,7 +338,7 @@ STextView::getCursorIndex (const SLocation& l)
   unsigned int si = textData.size(line);
   unsigned int nextExt = si;
   if (i<brk.size()) nextExt = brk[i];
-  bool endsline = false;
+  // SGC bool endsline = false;
 
   /* find the x position witihn the line */
   SCursorIndex cindex (line, currExt);
@@ -347,7 +356,7 @@ STextView::getCursorIndex (const SLocation& l)
 
      if (g.isEOP())
      {
-       endsline = true; /* never used */
+       // SGC endsline = true; 
      }
      else if (lm.x >= (int)begpos && lm.x < (int)midpos)
      {
@@ -526,8 +535,20 @@ STextView::internalRedraw (SCanvas *c, int x, int y,
        * (int) sane_index (lineSpan, line));
   SLocation lright(location.x+viewPort.x+(int)size.width, lleft.y);
 
-  for (unsigned int i=line; i<textData.size(); i++)
+  unsigned int i;
+
+  // expand +/- SD_PRE_EXPAND  lines by calling textData.size(i).
+  unsigned int start = (line > SD_PRE_EXPAND) ? line-SD_PRE_EXPAND : 0;
+  for (i=start; i<textData.size(); i++)
+  {
+    // dummy is never negative.
+    textData.size(i);
+    if (i>line+SD_PRE_EXPAND) break;
+  }
+
+  for (i=line; i<textData.size(); i++)
   { 
+    textData.size(i);
     setVisible (i);
     islr  = textData.isLR (i);
     unsigned int cs = islr 
@@ -590,7 +611,13 @@ STextView::drawParagraph (SCanvas* c, bool islr, unsigned int line,
      /* move the clipping area */
      if (br[currExt] == i)
      {
-       currExt++;
+       // printer can have same extent
+       while (br[currExt] == i)
+       {
+         currExt++;
+         if (currExt >= br.size()) break;
+       }
+
        lm.y = lineHeight * currExt + l.y;
        if (iswindow && clipw != 0 && cliph != 0)
        {
@@ -631,20 +658,41 @@ STextView::drawParagraph (SCanvas* c, bool islr, unsigned int line,
  * Set Syntax Hilight Mode
  */
 void
-STextView::setHighlightMode (SS_HighlightMode hlm)
+STextView::setSyntax (const SString& hlm)
 {
   highlightMode = hlm;
+  // these are kept for backward compatibility, hard coded.
+  if (hlm == "simple" || hlm == "simple-dark" || hlm == "none")
+  {
+    syntax.setSyntax ("");
+  }
+  else
+  {
+    syntax.setSyntax (hlm);
+  }
 }
 
 /**
  * Get Syntax Hilight Mode
  */
-STextView::SS_HighlightMode
-STextView::getHighlightMode () const
+const SString&
+STextView::getSyntax () const
 {
   return highlightMode;
 }
 
+void
+STextView::setSyntaxColors (const SSyntaxColors& attr)
+{
+  syntaxColors = attr;
+}
+
+const SSyntaxColors&
+STextView::getSyntaxColors () const
+{
+  return syntaxColors;
+}
+
 /**
  * Set WordWrap  Mode
  * @param pbm is true if line break is on
@@ -680,40 +728,72 @@ STextView::getWordWrap () const
  * Syntax Highlighting  system.
  * Added by Maarten van Gompel <proycon@anaproy.homeip.net>
  * Note:  this is a dumb system and merely colors single characters.
+ * For error highlighting, set forground to NONE and error to true.
  */
 void
-STextView::syntaxHighlight(STextIndex index, SPen* pen) const
+STextView::syntaxHighlight(STextIndex index, SPen* pen, bool *isError)
 {
- 
-  switch (getHighlightMode())
+  *isError = false;
+  if (highlightMode == "simple") 
   {
-  case HM_Simple:
-    if (textData.isDelimiter(index))
+    const SGlyph& g = textData.glyphAt (index);
+    if (g.isLetter ())
     {
-      pen->setForeground(SColor("CornflowerBlue"));
-      const SGlyph& g = textData.glyphAt (index);
-      if (g.isEOL()) (pen->setForeground(SColor("orange")));
     }
-    else if (textData.isNumber(index))
+    else if (g.isNumber())
     {
       pen->setForeground(SColor("orange"));
     }
-    return;
-  case HM_SimpleDark:
-    if (textData.isDelimiter(index))
+    else // not a letter, nor a number
     {
-      pen->setForeground(SColor("DeepSkyBlue4"));
-      const SGlyph& g = textData.glyphAt (index);
-      if (g.isEOL()) (pen->setForeground(SColor("orange4")));
+      pen->setForeground(SColor("CornflowerBlue"));
     }
-    else if (textData.isNumber(index))
+    if (getLigatureScriptCode (g.getChar()) == SD_AS_LITERAL) *isError = true; 
+  }
+  else if (highlightMode == "simple-dark") 
+  {
+    const SGlyph& g = textData.glyphAt (index);
+    if (g.isLetter ())
+    {
+    }
+    else if (g.isNumber())
     {
       pen->setForeground(SColor("orange4"));
     }
-    return;
-  case HM_HTML:
-  default:
-    break;
+    else // not a letter, nor a number
+    {
+      pen->setForeground(SColor("DeepSkyBlue4"));
+    }
+    if (getLigatureScriptCode (g.getChar()) == SD_AS_LITERAL) *isError = true; 
+  }
+  else if (highlightMode != "none" && highlightMode != "") 
+  {
+    
+    SSyntax::SS_Tag tag = syntax.getTagByTDI (index);
+    const SGlyph& g = textData.glyphAt (index);
+    if (getLigatureScriptCode (g.getChar()) == SD_AS_LITERAL)
+    {
+      *isError = true; 
+      tag = SSyntax::SD_CONTROL;
+      SColor c = syntaxColors.colors[(unsigned int) tag];
+      pen->setForeground(c);
+    }
+    // SD_ERROR and SD_NONE is preserving old color
+    else if (tag == SSyntax::SD_ERROR)
+    {
+      // SColor c = syntaxColors.colors[(unsigned int) SSyntax::SD_NONE];
+      // pen->setForeground(c);
+      *isError = true;
+    }
+    else if (tag == SSyntax::SD_NONE)
+    {
+      // preserve original color
+    }
+    else
+    {
+      SColor c = syntaxColors.colors[(unsigned int) tag];
+      pen->setForeground(c);
+    }
   }
   return;
 }
@@ -742,10 +822,13 @@ STextView::drawGlyph (SCanvas* c,
     p = rlpen;
   }
   unsigned int explevel = g.getExplicitLevel();
+  bool isError = false;
   if (g.selected)
   {
-     SColor fg = p.getForeground();
-     SColor bg = p.getBackground();
+     //SColor fg = p.getForeground();
+     //SColor bg = p.getBackground();
+     SColor fg = SColor("DeepSkyBlue4");
+     SColor bg = SColor("white");
      p.setForeground (bg);
      p.setBackground (fg);
      c->bitfill (fg, l.x, l.y, ext, lineHeight);
@@ -764,13 +847,13 @@ STextView::drawGlyph (SCanvas* c,
      p.setBackground (bg);
      c->bitfill (bg, l.x, l.y, ext, lineHeight);
   }
-  else
+  if (!g.selected) 
   { 
    /* 
     * Syntax highlighting, an addition 
     * by Maarten van Gompel <proycon@anaproy.homeip.net>
     */
-    syntaxHighlight(index, &p); /* change pen color if necessary */
+    syntaxHighlight(index, &p, &isError); /* change pen color if necessary */
   }
 
   if (!lineend && g.isEOP()) return;
@@ -793,15 +876,95 @@ STextView::drawGlyph (SCanvas* c,
     unsigned int base =  (lineAscent + h >= lineHeight) ?
          lineHeight -1 : lineAscent + h;
     /* construct a square */
-    c->bitfill (underlineColor, l.x, l.y + (int) base - h, w, h);
+    // changed in 2.8.2
+    //c->bitfill (underlineColor, l.x, l.y + (int) base - h, w, h);
+    c->bitfill (underlineColor, l.x, l.y + (int) base, w, h);
+  }
+  else if(isError)
+  {
+    SColor errColor = syntaxColors.colors[(unsigned int) SSyntax::SD_ERROR];
+    unsigned int w = ext;
+    unsigned int h = 3; // height occupies 3 pixels
+    unsigned int base =  (lineAscent + h >= lineHeight) ?
+         lineHeight - 1 : lineAscent + h;
+    int screenBase = l.y + (int) base;
+    SBinVector<int> x;
+    SBinVector<int> y;
+    for (unsigned i=0; i<w; i++)
+    {
+      x.append (l.x + i);
+      int state = ((l.x+i+clipx) % 6);
+      switch (state)
+      {
+      case 0: y.append (screenBase); break;
+      case 1: y.append (screenBase-1); break;
+      case 2: y.append (screenBase-1); break;
+      case 3: y.append (screenBase); break;
+      case 4: y.append (screenBase+1); break;
+      case 5: y.append (screenBase+1); break;
+      default: y.append (screenBase);
+      }
+    }
+    if (x.size() > 0)
+    {
+      c->bitpoints (errColor, x.array(), y.array(), x.size());
+    }
   }
 }
 
 /**
- * This is called by the STextData
+ * This can be called by the STextData and SSyntax.
  */
 void
-STextView::textChanged (void* src, const STextDataEvent& event)
+STextView::textChanged (void* src, const STextDataEvent& unparsedEvent)
+{
+  if (src == &syntax && unparsedEvent.attribute)
+  {
+    STextIndex tb = unparsedEvent.start;
+    STextIndex te = unparsedEvent.remaining;
+    // convert te back to text data coords
+    te.line = te.line >= textData.size() ? 0 : textData.size() - te.line;
+    // Filter out visible range
+    unsigned int firstVisible = getLineIndex (0);
+    int height = (int) getSize().height;
+    int width = (int) getSize().width;
+    unsigned int lastVisible = getLineIndex (height);
+    // filter out nont visible portion
+    if (te.line < firstVisible) return;
+    if (tb.line > lastVisible+1) return;
+
+    if (tb.line < firstVisible) tb.line = firstVisible;
+    tb.index = 0;
+    // non-inclusive
+    if (te.line > lastVisible) te.line = lastVisible+1;
+    te.index = 0;
+    // calculate the screen index, and do a redraw 
+    if (te.line > textData.size()) te.line = textData.size();
+    if (tb.line > textData.size()) tb.line = textData.size();
+    int bcoord = (int) lineHeight * sane_index (lineSpan, tb.line); 
+    int ecoord = (int) lineHeight * sane_index (lineSpan, te.line); 
+    // as we scroll down viewPort.y becomes negative.
+    // y coord of top of starting line.
+    bcoord += (location.y + viewPort.y) - 1;
+    // y coord of top of ending line.
+    ecoord += (location.y + viewPort.y) - 1;
+    SWindow* w = getWindow ();
+    if (w && ecoord > bcoord && bcoord < height && ecoord > 0)
+    {
+      /* request a redraw and clear the whole area + overdraw */
+      w->redraw (true, 0, bcoord, width, ecoord-bcoord + 2);
+//fprintf (stderr, "redraw 0,%d %u,%u\n", bcoord, getSize().width, ecoord-bcoord);
+//fprintf (stderr, "tb=%u te=%u\n", tb.line, te.line);
+    }
+  }
+  else
+  {
+    textChangedInternal (src, unparsedEvent);
+  }
+}
+
+void
+STextView::textChangedInternal (void* src, const STextDataEvent& event)
 {
   /* The whole text has been cleared */
   if (textData.size()==0)
@@ -827,11 +990,10 @@ STextView::textChanged (void* src, const STextDataEvent& event)
 
   SV_UINT oldbreaks;
 
-
   if (tb.line == te.line && tb.line < oldsize && tb.line < breaks.size())
   {
     /* This is still the old breaks */
-    oldbreaks = breaks[tb.line]; 
+    oldbreaks = breaks[tb.line];
   }
 
   /* change in text contents */
@@ -1244,8 +1406,8 @@ STextView::wrapAndPosition (unsigned int line, SH_UINT* cache)
   unsigned int i;
   SV_UINT b;
   bool wrapNext = false;
+  bool wrapPage = false;
   unsigned int lastbreak = 0;
-
   /* go through the text in logical order */
   for (i=0; i<textData.size(line); i++)
   {
@@ -1285,18 +1447,18 @@ STextView::wrapAndPosition (unsigned int line, SH_UINT* cache)
     ae += ce;
     logical[i] = ce;
 
-    if (multiline && le > size.width && i > 0)
+    if (multiline && le > size.width && i > 0 && !wrapNext)
     {
        if (g.isTab())
        {
          /* nothing to do. we break here */
        }
        /* we might want to wrap earlier */
-       else if (isWordWrapOn && !textData.isDelimiter (STextIndex (line, i-1)))
+       else if (isWordWrapOn && !textData.canWrap (STextIndex (line, i-1)))
        {
          unsigned int oldae = ae;
          unsigned int oldi = i;
-         while (i>lastbreak && !textData.isDelimiter (STextIndex (line, i-1)))
+         while (i>lastbreak && !textData.canWrap (STextIndex (line, i-1)))
          {
            ae -= logical[i];
            i--;
@@ -1314,11 +1476,42 @@ STextView::wrapAndPosition (unsigned int line, SH_UINT* cache)
     }
     else if (wrapNext)
     {
-       le = logical[i];
-       b.append (i);
-       lastbreak = i;
+      le = logical[i];
+      lastbreak = i;
+      if (wrapPage)
+      {
+        if (i == 1 && line == 0) // first line, first char is a FF
+        {
+          b.append (i);
+        }
+        else if (i > 1) // we have something on the line
+        {
+          //const SGlyph& gp = textData.glyphAt (STextIndex (line, i-2));
+          // the one before FF is an FF too
+          b.append (i);
+        }
+        unsigned int currSpan =  (line == 0 || lineSpan.size() < line-1)
+          ? 0 : lineSpan[line-1];
+
+        // how many more we need to add to reach top?
+        unsigned int lh = (lineHeight == 0) ? 1 : lineHeight;
+        unsigned int linesPerPage = printerPageSize / lh;
+        while (((currSpan + b.size()) % linesPerPage) != 0)
+        {
+           b.append (i);
+        }
+        
+      }
+      else
+      {
+        b.append (i);
+      }
     }
     wrapNext = (multiline && g.isEOL() && !g.isEOP());
+    if (printerPageSize != 0 && wrapNext)
+    {
+      wrapPage = g.isFF();
+    }
   }
   /* now b contains the logical positions where the glyph should start at 0 */
   b.append (textData.size(line));
@@ -1341,7 +1534,7 @@ STextView::wrapAndPosition (unsigned int line, SH_UINT* cache)
   for (i=0; i<textData.size(line); i++)
   {
     /* we use visual break here */
-    if (i == b[nextbreak])
+    while (nextbreak < b.size() && i == b[nextbreak])
     {
       nextbreak++;
       le = 0;
