@@ -27,18 +27,17 @@
 #include "swidget/SComponent.h"
 #include "swindow/SFont.h"
 #include "swindow/SWindow.h"
+#include "swindow/SSyntaxColors.h"
+#include "stoolkit/syntax/SSyntax.h"
 
 typedef SBinHashtable<SS_UINT> SH_UINT;
 
-
 /**
  * Provide a widget view to STextData
  */
 class STextView : public SComponent, STextDataListener
 {
 public:
-  enum SS_HighlightMode { HM_None, HM_Simple, HM_SimpleDark, HM_HTML};
-
   STextView (void);
   STextView (const SString& utf8);
   virtual ~STextView ();
@@ -79,8 +78,11 @@ public:
 
   unsigned int getDocumentHeight () const;
 
-  void setHighlightMode (SS_HighlightMode hlMode);
-  SS_HighlightMode getHighlightMode () const;
+  void setSyntax (const SString& hlMode);
+  const SString&  getSyntax () const;
+
+  void setSyntaxColors (const SSyntaxColors& attr);
+  const SSyntaxColors&  getSyntaxColors () const;
 
   void setWordWrap (bool lbm);
   bool getWordWrap () const;
@@ -93,13 +95,31 @@ public:
   void setClippingArea (int x, int y, 
           unsigned int width, unsigned int height);
 
+  void addSyntaxListener (SSyntaxListener* _listener)
+     { syntax.addSyntaxListener (_listener); }
+
   void setEditable (bool editable);
   void setText (const SString& text);
 
+  SString getSyntaxName () const
+  { 
+     return syntax.getParser ();
+  }
+  SString getHighlightName () const
+  { 
+     return highlightMode;
+  }
+
+  void setPrinterPageSize (unsigned int _ps)
+   { printerPageSize = _ps; }
+
 private:
-  SS_HighlightMode  highlightMode;
+  SSyntaxColors     syntaxColors;
+  SString           highlightMode;
+  SSyntax           syntax;
   bool              isWordWrapOn;
   bool              isEditable;
+  unsigned int      printerPageSize;
 
   SCursorIndex moveCursor (const SCursorIndex& ci, bool up);
    
@@ -112,6 +132,8 @@ private:
   void setPen();
   /* STextDataListener */
   void textChanged (void* src, const STextDataEvent& event);
+  void textChangedInternal (void* src, const STextDataEvent& event);
+
   unsigned int drawParagraph (SCanvas* c, bool islr, unsigned int line, 
        const SLocation& l, const SLocation& lb, 
        const SLocation& le, bool iswindow=false);
@@ -121,7 +143,7 @@ private:
 
 
 
-  void syntaxHighlight(STextIndex index, SPen* pen) const;
+  void syntaxHighlight(STextIndex index, SPen* pen, bool* isError);
   bool checktext(STextIndex index, const char* checkstring);
 
 
