@@ -132,4 +132,18 @@ SAwt::implemented()
   return (delegate!=0);
 }
 
+bool
+SAwt::hasGUI()
+{
+  bool ret = false;
+#ifdef USE_WINAPI
+  ret = true;
+#else
+# if USE_X11
+  ret = true;
+# endif
+#endif
+  return ret;
+}
+
 SAwtImpl* SAwt::delegate = 0;
