@@ -80,6 +80,8 @@ SFont::SFont (void) : name("default"), xlfd (SD_XLFD_ANY)
         "raavi.ttf:guru,shruti.ttf:gujr,"
         "arialuni.ttf,cyberbit.ttf,"
         "code2000.ttf,code2001.ttf:unicode:RL,arial.ttf,"
+        //"rovasSMP.ttf," // Hosszu Gabor
+        //"oldhunSMP.ttf," // Michael Everson
         "yudit.ttf"
     );
     SFontImplVector list;
@@ -245,7 +247,10 @@ mapFB (const SGlyph& glyph)
   {
     if (glyph.getChar() == SD_CD_CR) return SFontFB::CR;
     if (glyph.getChar() == SD_CD_LF) return SFontFB::LF;
+
     if (glyph.getChar() == SD_CD_LS) return SFontFB::LS;
+    if (glyph.getChar() == SD_CD_FF) return SFontFB::FF;
+
     if (glyph.getChar() == SD_CD_PS) return SFontFB::PS;
     if (glyph.getChar() == SD_CD_TAB) return SFontFB::TAB;
     if (glyph.getChar() == SD_CD_LRM) return SFontFB::LRM;
@@ -331,18 +336,19 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
 
     /* use mirrored glyphs for Old Hungarian, Old Italic */
     if (  (comp >= 0xEE00 && comp < 0xEE9F)
-        || (comp >= 0x10300 && comp <= 0x1032F))
+        || (comp >= 0x10300 && comp <= 0x1032F)
+        || (comp >= 0x10c00 && comp <= 0x10fff)
+        || scriptcode == SD_ROVASIRAS || scriptcode == SD_PUA_ROVAS)
     {
-      /* Try to use lr and rl attributes */
+      /* Try to use lr and rl attributes. This is the non-mirrored dwaring. */
       for (i=0; i<fontVector.size(); i++)
       {
         SFontImpl im = fontVector[i];
-        if (im.isLR() && !glyph.isLR()) continue;
-        if (im.isRL() && glyph.isLR()) continue;
-        /* neutral = lr */
-        if (!im.isLR() && !im.isRL() && !glyph.isLR()) continue;
+        // Skip cases where font is strongly the opposite direction.
+        if (im.needSoftMirror (comp, glyph.isLR())) continue;
+        
         /* this is for better positioning of diacritical marks. */
-        if (im.draw (canvas, pen, m, comp)) 
+        if (im.draw (canvas, pen, m, comp, glyph.isLR())) 
         {
           im.width (comp, &baseWidth);
           baseOK=true;
@@ -363,7 +369,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
           {
             mm.t0 = m.t0 + currw;
           }
-          im.draw (canvas, pen, mm, comp); 
+          im.draw (canvas, pen, mm, comp, glyph.isLR()); 
           baseWidth = currw;
           baseOK = true;
           break;
@@ -377,7 +383,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
       for (i=0; i<fontVector.size(); i++)
       {
         SFontImpl im = fontVector[i];
-        if (im.draw (canvas, pen, m, comp)) 
+        if (im.draw (canvas, pen, m, comp, glyph.isLR())) 
         {
           im.width (comp, &baseWidth);
           baseOK = true;
@@ -399,7 +405,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
       for (unsigned int i=0; i<fontVector.size(); i++)
       {
         SFontImpl im = fontVector[i];
-        if (im.draw (canvas, pen, m, orig)) 
+        if (im.draw (canvas, pen, m, orig, glyph.isLR())) 
         {
           im.width (orig, &baseWidth);
           baseOK = true;
@@ -572,7 +578,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
         {
           mo.translate (translatex, (double)0.0);
           SFontImpl im = fontVector[indeces[i]];
-          im.draw (canvas, pen, mo, decomp[i]);
+          im.draw (canvas, pen, mo, decomp[i], glyph.isLR());
         }
       }
     }
@@ -599,7 +605,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
       for (unsigned int i=0; i<fontVector.size(); i++)
       {
         SFontImpl im = fontVector[i];
-        if (im.draw (canvas, pen, m, orig)) 
+        if (im.draw (canvas, pen, m, orig, glyph.isLR())) 
         {
           im.width (orig, &baseWidth);
           baseOK = true;
@@ -656,7 +662,7 @@ SFont::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
           }
         }
         /* Draw it finally... */
-        if (im.draw (canvas, pen, mc, decomp[i])) 
+        if (im.draw (canvas, pen, mc, decomp[i], glyph.isLR())) 
         {
            break;
         }
