@@ -24,6 +24,10 @@
  * http://www.brockmeiers.de/fonts/
  */
 
+#ifdef __GNUC__
+#pragma GCC diagnostic ignored "-Wnarrowing"
+#endif 
+
 #define SD_BOXR 10
 #define SD_BWIDTH (fb_width*SD_BOXR/400)
 
@@ -92,6 +96,34 @@ SD_BEGIN_GLYPH, 721, /*  width */
  SD_CLOSE_PATH,
 SD_END_GLYPH
 };
+
+/* line separator */
+short fb_ls[] =
+{
+SD_BEGIN_GLYPH, 721, /*  width */ 
+ SD_MOVE_TO,  700, 300,
+ SD_LINE_TO,  700, 0,
+ SD_LINE_TO,  100, 0,
+ SD_LINE_TO,  100, 50,
+ SD_LINE_TO,  300, 50,
+ SD_LINE_TO,  650, 50,
+ SD_LINE_TO,  650, 300,
+ SD_CLOSE_PATH,
+SD_END_GLYPH
+};
+
+/* form feed */
+short fb_ff[] =
+{
+SD_BEGIN_GLYPH, 721, /*  width */ 
+ SD_MOVE_TO,  700, 300,
+ SD_LINE_TO,  700, 0,
+ SD_LINE_TO,  100, 0,
+ SD_LINE_TO,  100, 300,
+ SD_CLOSE_PATH,
+SD_END_GLYPH
+};
+
 /* crlf */
 short fb_crlf[] =
 {
@@ -129,21 +161,6 @@ SD_BEGIN_GLYPH, 721, /*  width */
 SD_END_GLYPH
 };
 
-short fb_ls[] =
-{
-SD_BEGIN_GLYPH, 721, /*  width */ 
- SD_MOVE_TO,  500, 700,
- SD_LINE_TO,  500, 0,
- SD_LINE_TO,  300, 0,
- SD_LINE_TO,  300, -100,
- SD_LINE_TO,  50, 25,
- SD_LINE_TO,  300, 150,
- SD_LINE_TO,  300, 50,
- SD_LINE_TO,  450, 50,
- SD_LINE_TO,  450, 700,
- SD_CLOSE_PATH,
-SD_END_GLYPH
-};
 
 /* Paragraph Separator */
 short fb_ps[] =
@@ -898,6 +915,9 @@ SFontFB::signDraw (SCanvas* canvas, const SPen& pen,
   case LS:
     drawOne (canvas, f, fb_ls);
     break;
+  case FF:
+    drawOne (canvas, f, fb_ff);
+    break;
   case PS:
     drawOne (canvas, f, fb_ps);
     break;
@@ -1031,7 +1051,7 @@ void
 SFontFB::drawOne (SCanvas* canvas, const SS_Matrix2D& m, short* guide)
 {
   unsigned int i=0;
-  short wid;
+  //SGC short wid;
   double x, y;
   double x0, y0;
   double x1, y1;
@@ -1041,7 +1061,8 @@ SFontFB::drawOne (SCanvas* canvas, const SS_Matrix2D& m, short* guide)
     switch (guide[i])
     {
     case  SD_BEGIN_GLYPH:
-       wid = guide[i+1]; i=i+2;
+       //SGC wid = guide[i+1];
+       i=i+2;
        break;
     case  SD_MOVE_TO:
        x = m.x0 * double (guide[i+1]) + m.y0 * double (guide[i+2]+fb_doffset) + m.t0;
