@@ -36,7 +36,7 @@ class SFontFB
 public:
   SFontFB (void); 
   virtual ~SFontFB (); 
-  enum SIGN { CR, LF, CRLF, LFCR, PS, LS, TAB, CTRL, LRM, RLM, FB_ZWNJ, FB_ZWJ };
+  enum SIGN { CR, LF, CRLF, LFCR, PS, LS, FF, TAB, CTRL, LRM, RLM, FB_ZWNJ, FB_ZWJ };
 
   void draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& matrix, 
     SS_UCS4 g);
