@@ -30,6 +30,7 @@
 #include "stoolkit/SHashtable.h"
 #include "stoolkit/SExcept.h"
 #include "stoolkit/SUtil.h"
+#include "stoolkit/SCluster.h"
 
 static SStringVector fontSearchPath(
    "/,fonts,../fonts,/etc/fonts,/usr/share/yudit/fonts");
@@ -526,7 +527,7 @@ SFontImpl::scale (double x, double y)
  */
 bool
 SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m, 
-  SS_UCS4 uch)
+  SS_UCS4 uch, bool isLRContext)
 {
   if (!delegate) return 0;
 
@@ -546,6 +547,7 @@ SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
         /* FIXME: you should in fact use an instance counter. */
         key.append ((long)(sd.x0*16000.0)); /* scale-x */
         key.append ((long)(sd.y1*16000.0)); /* scale-y */
+        if (!isLRContext) key.append ("R");
         double offsetX = 0.0;
         double offsetY = 0.0;
         /* for better positioning of diacritical marks */
@@ -559,7 +561,7 @@ SFontImpl::draw (SCanvas* canvas, const SPen& pen, const SS_Matrix2D& m,
         if (!canvas->newpath (m.t0+offsetX, m.t1+offsetY, key))
         {
           /* not cached yet */
-          d->u.ttf->draw (canvas, sd, uch);
+          d->u.ttf->draw (canvas, sd, uch, isLRContext);
         }
         canvas->fill (pen);
         return true;
@@ -805,3 +807,17 @@ SFontImpl::clone () const
   CHECK_NEW (ret);
   return ret;
 }
+
+bool
+SFontImpl::needSoftMirror (SS_UCS4 uch, bool isLRContext) const
+{
+  // Sort out strong font types.
+  if (isLR() && !isLRContext) return true;
+  if (isRL() && isLRContext) return true;
+  if (isLR() || isRL()) return false;
+
+  if (getPUARovasType (uch) > 0) return !isLRContext;
+  if (getRovasType (uch) > 0) return isLRContext;
+  // rest is LR, need mirroring in RL context.
+  return !isLRContext;
+}
