@@ -51,7 +51,7 @@ public:
   void   scale (double x, double y);
 
   bool draw (SCanvas* canvas, const SPen& pen, 
-       const SS_Matrix2D& matrix, SS_UCS4 uch);
+       const SS_Matrix2D& matrix, SS_UCS4 uch, bool isLRContext);
   bool width (SS_UCS4 uch, double* width);
 
   double width () const;
@@ -67,6 +67,7 @@ public:
   void setAttributes (const SProperties& properties);
   bool isLR () const { return lrFont; }
   bool isRL () const { return rlFont; }
+  bool needSoftMirror (SS_UCS4 uch, bool isLRContext) const;
 
   /* set the base character for better glyph positioning */
   static void setBase(SS_UCS4 base);
