@@ -768,7 +768,8 @@ SFontTTF::getGlyphWidth (SS_GlyphIndex glyph)
  * @return true if drawn
  */
 bool
-SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, SS_UCS4 _uch)
+SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, SS_UCS4 _uch, 
+  bool isLRContext)
 {
   if (!isOK()) return false;
   SV_GlyphIndex gi;
@@ -819,12 +820,27 @@ SFontTTF::draw (SCanvas* canvas, const SS_Matrix2D& matrix, SS_UCS4 _uch)
   { 
     for (unsigned int i=0; i<gi.size(); i++)
     {
-      /* do not draw them */
-      SS_GlyphIndex g = gi[i];
+      bool isRovas = getLigatureScriptCode (_uch) == SD_ROVASIRAS
+           || getLigatureScriptCode (_uch) == SD_PUA_ROVAS;
+      // For failsafe we preserve old behaviour for non SD_ROVASIRAS
+      SS_GlyphIndex g = 
+         (!isRovas  
+          || (isLRContext && mo.x0 >= 0.0)
+          || (!isLRContext && mo.x0 < 0.0)) ? gi[i] : gi[gi.size()-i-1];
+
       if (g != SD_G_INDIC_ZWJ && g != SD_G_INDIC_ZWNJ)drawGlyph (canvas, mo, g);
       double nwidth = widthGlyph (mo, g);
       if (nwidth < 0) nwidth = -nwidth;
-      mo.translate (nwidth, 0.0);
+      // For failsafe we preserve old behaviour for non SD_ROVASIRAS
+      // Mirroring (mo.x0 < 0) will reverse the glyph order.
+      if (!isRovas || mo.x0 >= 0.0)
+      {
+        mo.translate (nwidth, 0.0);
+      }
+      else
+      {
+        mo.translate (-nwidth, 0.0);
+      }
     }
   }
   return true;
@@ -1205,7 +1221,77 @@ SFontTTF::findGlyphs (SS_UCS4 in, SV_GlyphIndex* out)
   unsigned int scriptcode = getLigatureScriptCode (lig);
   /* no support yet for this monster */
 
-  if (scriptcode == SD_COMBINING_LIGATURE)
+  if ( (scriptcode == SD_ROVASIRAS  || scriptcode == SD_PUA_ROVAS)
+       
+       && (liglen=getLigatureUnicode(lig, 0)) > 0)
+  {
+     if (fontencoding.size()!=0)
+     {
+        SString chc; char2Glyphs.put (key, chc);
+        return false;
+     }
+     SS_UCS4* chars =  new SS_UCS4[liglen];
+     CHECK_NEW (chars);
+     getLigatureUnicode (lig, chars);
+     SS_GlyphIndex* gi = new SS_GlyphIndex [liglen];
+     CHECK_NEW (gi);
+
+     unsigned int count = 0;
+     unsigned int i;
+     for (i=0; i<liglen; i++)
+     {
+        gi[count] = findGlyph (chars[i], false);
+        if (gi[count] == 0) 
+        {
+            if (chars[i] != 0x200d) 
+            {
+               SString chc; char2Glyphs.put (key, chc);
+               delete chars;
+               delete gi;
+               return false;
+            }
+        } 
+        else
+        {
+          count++;
+        }
+     }
+     if (count == 1)
+     {
+       for (i=0; i<count; i++) out->append (gi[i]); 
+       SString chc ((char*)gi, count * sizeof (SS_GlyphIndex));
+       char2Glyphs.put (key, chc);
+       delete chars;
+       delete gi;
+       return true;
+     }
+    
+     SS_GlyphIndex lig = 0;
+
+     // I dont know yet what feature is best. 
+
+     // We dont know the script code, so we pass 0.
+     // We dont know if it should be alig, dlig or rlig. 
+     // Use all that is available.
+     unsigned int ligs = getOTFLigature (0, "rlig,liga", &gi[0], count, &lig, 4);
+     while (ligs > 1 && ligs <= count) 
+     {
+       gi[0] = lig;
+       // ligs fell out.
+       for(i=1; i<count-ligs; i++) gi[i] = gi[i+ligs];
+       count = count-ligs+1;
+       ligs = getOTFLigature (0, "rlig,liga", &gi[0], count, &lig, 4);
+     }
+
+     for (i=0; i<count; i++) out->append (gi[i]); 
+     SString chc ((char*)gi, count * sizeof (SS_GlyphIndex));
+     char2Glyphs.put (key, chc);
+     delete chars;
+     delete gi;
+     return true;
+     
+  }
+  else if (scriptcode == SD_COMBINING_LIGATURE)
   {
     /* never comes here */
     SString chc;
