@@ -1,10 +1,7 @@
 /** 
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 2003  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2002  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2001  Gaspar Sinai <gsinai@yudit.org>  
- *  GNU Copyright (C) 2000  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -29,7 +26,7 @@
 #include "stoolkit/SUniqueID.h"
 
 /**
- * @author: Gaspar Sinai <gsinai@yudit.org>
+ * @author: Gaspar Sinai <gaspar@yudit.org>
  * @version: 2000-04-23
  * This is the abstract widget toolkit font package
  */
@@ -45,7 +42,7 @@ public:
        int origox, int origoy, 
        unsigned int width, unsigned int height);
   virtual ~SImage();
-  const SUniqueID& SImage::getID() const;
+  const SUniqueID& getID() const;
   void compress();
   /* After fill, return the shade of white  */
   unsigned int getShades () const;
@@ -56,7 +53,7 @@ public:
   void setOrigoX (int x);
   void setOrigoY (int y);
   SS_WORD32 getShade (int x, int y) const;
-  virtual SObject* SImage::clone () const;
+  virtual SObject* clone () const;
 
   /* USed by cache device to indicate the offset */
   int  px;
