@@ -25,10 +25,12 @@
  * Bitmap font support
  */
 #ifndef USE_WINAPI 
-#include <X11/Xlib.h>
-#include <X11/Xutil.h>
-#include "swindow/SAwt.h"
-#include "swindow/sx11/SX11Impl.h"
+# if USE_X11
+#  include <X11/Xlib.h>
+#  include <X11/Xutil.h>
+#  include "swindow/SAwt.h"
+#  include "swindow/sx11/SX11Impl.h"
+# endif
 #endif
 
 /**
@@ -208,11 +210,29 @@ SPostscript::open (bool background)
   SStringVector head("%!PS-Adobe-3.0");
   SString bb("%%BoundingBox: ");
 
-  bb.print (widthMargin); bb.append (" ");
-  bb.print (heightMargin); bb.append (" ");
-  bb.print (width - widthMargin); bb.append (" ");
-  bb.print (height - heightMargin); 
-  head.append (bb);
+  switch (orientation)
+  {
+  case LANDSCAPE:
+ 
+        bb.print (heightMargin); bb.append (" ");
+        bb.print (widthMargin); bb.append (" ");
+        bb.print (height - heightMargin); bb.append (" ");
+        bb.print (width - widthMargin);
+        head.append (bb);
+ 
+    break;
+ 
+  case PORTRAIT:
+  default:
+ 
+        bb.print (widthMargin); bb.append (" ");
+        bb.print (heightMargin); bb.append (" ");
+        bb.print (width - widthMargin); bb.append (" ");
+        bb.print (height - heightMargin);
+        head.append (bb);
+ 
+    break;
+  }
 
   head.append ("%%Title: Untitled");
   SString creator ("%%Creator: yudit "); creator.append (SD_YUDIT_VERSION);
@@ -262,8 +282,16 @@ SPostscript::open (bool background)
   default:
      m.append ("A4 ");
   }
-  m.print (width); m.append (" ");
-  m.print (height); m.append (" 0 () ()");
+  if (orientation == LANDSCAPE)
+  {
+    m.print (height); m.append (" ");
+    m.print (width); m.append (" 0 () ()");
+  }
+  else 
+  {
+    m.print (width); m.append (" ");
+    m.print (height); m.append (" 0 () ()");
+  }
   head.append (m);
   // windows gs goes crazy if I uncomment this.
   //head.append ("");
@@ -329,7 +357,25 @@ SPostscript::newPage()
     }
     currentPage++;
     char buff[128];
-    sprintf (buff, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
+ 
+  switch (orientation)
+  {
+  case LANDSCAPE:
+ 
+          sprintf (buff, "\n%%%%Page: %u %u\nsave\n90 rotate\n0 -%d.000 translate\n", pages, currentPage, height);
+ 
+    break;
+ 
+  case PORTRAIT:
+  default:
+ 
+          sprintf (buff, "\n%%%%Page: %u %u\nsave\n", pages, currentPage);
+ 
+    break;
+  }
+ 
+ 
+ 
     writeString (buff);
   }
 }
@@ -454,6 +500,7 @@ SPostscript::newpath (double _x, double _y, const SString& _id)
       int iy =  (int) (1000.0 * ((double)height-_y));
       sprintf (arr, "gsave %s%d.%03d %s%d.%03d translate newpath\n", 
         SG(ix), ix/1000, SM(ix), SG(iy), iy/1000, SM(iy));
+      // BUG-2011-08-04 Don't write. This happens between U+0000...U+001F
       //writeString (arr);
       return true;
     }
@@ -500,7 +547,9 @@ SPostscript::fill ()
   }
   else 
   {
-    if (cache.get (cacheID) == 0) writeString ("fill grestore\n");
+    // BUG-2011-08-04 Don't write. This happens between U+0000...U+001F
+    // Originally only newpath was commented out.
+    // if (cache.get (cacheID) == 0) writeString ("fill grestore\n");
   }
 }
 
@@ -858,10 +907,45 @@ void
 SPostscript::bitpoints (const SColor& fg, const int* x, const int* y, 
          unsigned int _size)
 {
-  for (unsigned int i=0; i<_size; i++)
+  if (_size<=0) return;
+  SString key("BPS");
+  unsigned int i;
+  for (i=0; i<_size; i++)
   {
-    bitpoint (fg, x[i], y[i]);
+    key.append (SString(((char*) &x[i]), sizeof(int)));
+    key.append (SString(((char*) &y[i]), sizeof(int)));
   }
+  if (newpath (x[0], y[0], key))
+  {
+    fill ();
+    return;
+  }
+  for (i=0; i<_size; i++)
+  {
+    double squareX[4];
+    double squareY[4];
+    double delta = 1.0;
+
+    squareX[0] = (double)x[i];
+    squareY[0] = (double)y[i];
+
+    squareX[1] = squareX[0] + delta;
+    squareY[1] = squareY[0];
+
+    squareX[2] = squareX[0] + delta;
+    squareY[2] = squareY[0] + delta;
+
+    squareX[3] = squareX[0];
+    squareY[3] = squareY[0] + delta;
+
+    moveto (squareX[0], squareY[0]);
+    lineto (squareX[1], squareY[1]);
+    lineto (squareX[2], squareY[2]);
+    lineto (squareX[3], squareY[3]);
+    closepath();
+  }
+  fill ();
+  isUgly = true;
 }
 
 bool
