@@ -1,7 +1,7 @@
 /**
  *  Yudit Unicode Editor Source File
  *
- *  GNU Copyright (C) 1997-2005  Gaspar Sinai <gsinai@yudit.org>  
+ *  GNU Copyright (C) 1997-2006  Gaspar Sinai <gaspar@yudit.org>  
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License, version 2,
@@ -30,7 +30,7 @@
 /*!
  * \file SScriptProcessor.cpp
  * \brief Script Processing routines.
- * \author: Gaspar Sinai <gsinai@yudit.org>
+ * \author: Gaspar Sinai <gaspar@yudit.org>
  * \version: 2000-04-23
  */
 
@@ -1105,8 +1105,16 @@ SScriptProcessor::gindex()
   for (unsigned i=0; i<m_in.size(); i++)
   {
     index = m_font->gindex (m_in[i]);
-    // allow zwj missing.
-    if (index ==0 && m_in[i]!=0x200c && m_in[i]!=0x200d) ret = false; 
+    //
+    // allow ZWJ ZWNJ even though they are missing from font.
+    //
+    if (index == 0)
+    {
+      if (m_in[i] != 0x200c && m_in[i] != 0x200d)
+      {
+        ret = false;
+      }
+    }
     m_out.append (index);
   }
   return ret;
@@ -1285,12 +1293,18 @@ SScriptProcessor::findBaseConsonant()
     typeFirst = SRendClass::get (m_in[0]);
     if (typeFirst == SRendClass::Cbase 
       || typeFirst == SRendClass::Cpost
+      || typeFirst == SRendClass::Cdead
       || typeFirst == SRendClass::Cbelow)
     {
       lastConsonant = 1;
     }
   }
 
+  // Khanda Ta Treat it just like Ta
+  if (typeFirst == SRendClass::Cdead) 
+  {
+    typeFirst = SRendClass::Cbase;
+  }
   m_out_type.clear();
   m_out_type.append (typeFirst);
 
@@ -1303,9 +1317,15 @@ SScriptProcessor::findBaseConsonant()
   bool has_vo = false;
   bool has_mpost = false;
   unsigned int i;
+  // Initialize m_out_type.
   for (i=1; i<m_in.size(); i++)
   {
     SRendClass::RType type = SRendClass::get (m_in[i]);
+    // Khanda Ta Treat it just like Ta
+    if (type == SRendClass::Cdead) 
+    {
+      type = SRendClass::Cbase;
+    }
 
     // Creph and Cra can be only the first consonant 
     // This cycle starts with 1.
@@ -1417,6 +1437,7 @@ SScriptProcessor::findBaseConsonant()
 
   // Makes sure everything is ordered properly.
   bool post = false; // everything will be ppst after a post.
+
   for (i=0; i<m_out_type.size(); i++)
   {
     SRendClass::RType type = m_out_type[i];
@@ -1637,7 +1658,6 @@ SScriptProcessor::reorderStraight()
 void
 SScriptProcessor::reorderIndic ()
 {
-
   // 1. Find last consonant. Return value is last+1
   unsigned int last_c = findLastConsonant();
   unsigned int i;
@@ -1760,7 +1780,7 @@ SScriptProcessor::reorderIndic ()
           }
         }
       }
-#if 0
+#if DEBUG_ALL
       SString str ("reorder-add-");
       str.append (SRendClass::string(gclass));
       str.append ((char)0);
