@@ -78,58 +78,69 @@ SX11Window::setPixmapCacheOn (bool on)
 
 SX11Window::SX11Window(const SString& n, SX11Impl* _impl, long _id) : name (n),
  inputMethod (_impl, _id, SD_YUDIT_EVENT_MASK),
- pen (SColor(0), SColor(0xffffffff)),
- background (_impl, SS_WORD32(0xffffffff))
+ background (SX11Color(_impl, SS_WORD32(0xffffffff))),
+ pen (SColor(0), SColor(0xffffffff))
 {
+  impl = _impl;
+  dbindex = 0;
+
   isChild = 0;
   mapNotified = false;
-  id = _id;
-  impl = _impl;
-  impl->addWindow (id, this);
+  exposedOnce = false;
+  dbuffer[0].id = _id;
+  dbuffer[1].id = None;
+
+  dbuffer[0].gcFD = 0;
+
+  dbuffer[0].gcFG = (unsigned long) pen.getForeground().getValue();
+  impl->addWindow (dbuffer[0].id, this);
   XGCValues   gcv;
   gcv.foreground = SX11Color(impl, pen.getForeground()).getPixelValue();
   gcv.background = background.getPixelValue();
   clipped = false;
-  gcFD = 0;
-  gcFG = (unsigned long) pen.getForeground().getValue();
 
-  gc = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground, &gcv); 
-  gc_default = XCreateGC (impl->display, (Window)id, 0, &gcv);
+  dbuffer[0].gc = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground, &gcv); 
+
+  dbuffer[0].gc_default = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+      0, &gcv);
+
   gcv.function = GXand;
-  gc_draw_and = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground | GCFunction, &gcv); 
+  dbuffer[0].gc_draw_and = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground | GCFunction, &gcv); 
+
   gcv.function = GXor;
-  gc_draw_or = XCreateGC (impl->display, (Window)id, GCForeground
-    | GCBackground | GCFunction, &gcv); 
+  dbuffer[0].gc_draw_or = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+     GCForeground | GCBackground | GCFunction, &gcv); 
 
   setBackground (background);
   engine = 0;
   shown = false;
-  XSelectInput (impl->display, (Window) id, SD_YUDIT_EVENT_MASK);
+  XSelectInput (impl->display, (Window) dbuffer[0].id, SD_YUDIT_EVENT_MASK);
 }
 
 SX11Window::~SX11Window()
 {
-  XFreeGC (impl->display, gc);
-  XFreeGC (impl->display, gc_default);
-  XFreeGC (impl->display, gc_draw_and);
-  XFreeGC (impl->display, gc_draw_or);
-  impl->deleteWindow (id, this);
+  freeDoubleBuffer ();
+  XFreeGC (impl->display, dbuffer[0].gc);
+  XFreeGC (impl->display, dbuffer[0].gc_default);
+  XFreeGC (impl->display, dbuffer[0].gc_draw_and);
+  XFreeGC (impl->display, dbuffer[0].gc_draw_or);
+  impl->deleteWindow (dbuffer[0].id, this);
   if (engine) delete engine;
 }
 
 void
 SX11Window::show ()
 {
-  impl->show (id);
+  impl->show (dbuffer[0].id);
   shown = true;
 }
 
 void
 SX11Window::hide ()
 {
-  impl->hide (id);
+  impl->hide (dbuffer[0].id);
   shown = false;
 }
 
@@ -139,6 +150,12 @@ SX11Window::hide ()
 bool
 SX11Window::newpath (double _x, double _y, const SString& _id)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::newpath called without beginDoubleBuffer\n");
+    /* this will always return false */
+    return engine->newpath ((int)_x, (int)_y, "");
+  }
   if (engine ==0) engine = new SGEngine();
   if (isCacheOn)
   {
@@ -164,6 +181,11 @@ SX11Window::fill (const SPen& _pen)
   {
      pen = _pen;
   }
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::fill called without beginDoubleBuffer\n");
+    return;
+  }
 
   if (engine==0) return;
 
@@ -230,8 +252,9 @@ SX11Window::fill (const SPen& _pen)
       XPutPixel (ximage, x, y, rpvle); 
     }
   }
-  XPutImage (impl->display, (Window) id, gc_draw_and, ximage, 0, 0,
-     si->getOrigoX(), si->getOrigoY(), 
+  XPutImage (impl->display, (Drawable) dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_and, 
+     ximage, 0, 0, si->getOrigoX(), si->getOrigoY(), 
      imageWidth, imageHeight);
 
   /* Then blot the image */
@@ -245,7 +268,8 @@ SX11Window::fill (const SPen& _pen)
     }
   }
 
-  XPutImage (impl->display, (Window) id, gc_draw_or, ximage, 0, 0,
+  XPutImage (impl->display, (Drawable) dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_or, ximage, 0, 0,
      si->getOrigoX(), si->getOrigoY(), 
      imageWidth, imageHeight);
 
@@ -326,6 +350,11 @@ SX11Window::closepath()
 void 
 SX11Window::putImage (int _x, int _y, const SImage& im)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::putImage called without beginDoubleBuffer\n");
+    return;
+  }
   if (cacheCount > cacheSize)
   {
       fprintf (stderr, "SX11Window:: clearing cache (%u elements)\n",
@@ -403,9 +432,11 @@ fprintf (stderr, "\n");
   {
 //fprintf (stderr, "Using cache X11\n");
   }
-  XCopyArea(impl->display, mask, (Window)id, gc_draw_and, 0, 0, 
+  XCopyArea(impl->display, mask, (Drawable)dbuffer[dbindex].id, 
+     dbuffer[dbindex].gc_draw_and, 0, 0, 
      im.getWidth(), im.getHeight(), _x, _y);
-  XCopyArea(impl->display, pixmap, (Window)id, gc_draw_or, 0, 0, 
+  XCopyArea(impl->display, pixmap, (Drawable)dbuffer[dbindex].id, 
+    dbuffer[dbindex].gc_draw_or, 0, 0, 
     im.getWidth(), im.getHeight(), _x, _y);
 
   /* This should have the foreground and background in it */
@@ -449,6 +480,11 @@ fprintf (stderr, "\n");
 void
 SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createColoredPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
@@ -472,9 +508,11 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
       }
     }
 
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
       im.getWidth(), im.getHeight(), bits);
-    XPutImage (impl->display, xshape, gc_default, ximage, 0, 0, 0, 0, 
+    XPutImage (impl->display, xshape, dbuffer[dbindex].gc_default, 
+       ximage, 0, 0, 0, 0, 
       im.getWidth(), im.getHeight());
     *m = xshape;
   }
@@ -492,10 +530,11 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, (Drawable)dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+      ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   XDestroyImage (ximage);
@@ -511,6 +550,11 @@ SX11Window::createColoredPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 void
 SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createShadedPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
@@ -535,10 +579,12 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
         XPutPixel (ximage, x, y, rpvle); 
       }
     }
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
       im.getWidth(), im.getHeight(), bits);
 
-    XPutImage (impl->display, xshape, gc_default, ximage, 0, 0, 0, 0, 
+    XPutImage (impl->display, xshape, dbuffer[dbindex].gc_default, 
+      ximage, 0, 0, 0, 0, 
       im.getWidth(), im.getHeight());
     *m = xshape;
   }
@@ -574,10 +620,12 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, 
+     (Drawable) dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+     ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   for (i=0; i<shades; i++)
@@ -598,13 +646,19 @@ SX11Window::createShadedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 void
 SX11Window::createMaskedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::createMaskedPixmap called without beginDoubleBuffer\n");
+    return;
+  }
   int imageWidth = (int) im.getWidth();
   int imageHeight = (int) im.getHeight();
 
   /* First wipe out the shape */
   if (m != 0)
   {
-    Pixmap xshape = XCreatePixmap (impl->display, (Window) id, 
+    Pixmap xshape = XCreatePixmap (impl->display, 
+      (Drawable) dbuffer[dbindex].id, 
        im.getWidth(), im.getHeight(), 1);
     XGCValues   gcv;
     gcv.background = 0x0;
@@ -650,10 +704,12 @@ SX11Window::createMaskedPixmap (const SImage& im, Pixmap* p, Pixmap* m)
     }
   }
 
-  Pixmap xpixmap = XCreatePixmap (impl->display, (Window) id, 
+  Pixmap xpixmap = XCreatePixmap (impl->display, 
+     (Drawable) dbuffer[dbindex].id, 
      im.getWidth(), im.getHeight(), bits);
 
-  XPutImage (impl->display, xpixmap, gc_default, ximage, 0, 0, 0, 0, 
+  XPutImage (impl->display, xpixmap, dbuffer[dbindex].gc_default, 
+     ximage, 0, 0, 0, 0, 
      im.getWidth(), im.getHeight());
 
   XDestroyImage (ximage);
@@ -665,7 +721,8 @@ SX11Window::setBackground(const SColor &color)
 {
   SX11Color c(impl, color);
   background = c;
-  XSetWindowBackground (impl->display, (Window)id, background.getPixelValue());
+  XSetWindowBackground (impl->display, (Window)dbuffer[0].id, 
+     background.getPixelValue());
   pen = SPen(pen.getForeground(), color, pen.getLineWidth());
 }
 
@@ -711,12 +768,13 @@ SX11Window::popmatrix()
 void
 SX11Window::resize (unsigned int _width, unsigned int _height)
 {
-//  fprintf (stderr, "resizing from: %u %u to %u %u\n",
-// width, height, _width, _height);
+  //fprintf (stderr, "resizing from: %lx   %u %u to %u %u\n",
+   //(unsigned long) this, xwidth, yheight, _width, _height);
+
   if (getWidth() == _width && getHeight() == _height)return;
   if (_width == 0) _width = 1;
   if (_height == 0) _height = 1;
-  XResizeWindow (impl->display, (Window) id, _width, _height);
+  XResizeWindow (impl->display, (Window) dbuffer[0].id, _width, _height);
   setSize(_width, _height);
 }
 
@@ -724,7 +782,7 @@ void
 SX11Window::move (int _x, int _y)
 {
   if (getPositionX() == _x && getPositionY() == _y)return;
-  XMoveWindow (impl->display, (Window) id, _x, _y);
+  XMoveWindow (impl->display, (Window) dbuffer[0].id, _x, _y);
   setPosition(_x, _y);
 }
 /**
@@ -739,26 +797,34 @@ void
 SX11Window::bitfont (const SPen& _pen, double x, double y, 
   void* native, char* data, unsigned int len)
 {
-  if (gcFD != (unsigned long) native)
+  if (dbuffer[dbindex].id == None)
   {
-    XSetFont (impl->display, gc, (Font) native);
-    gcFD  = (unsigned long) native;
+    fprintf (stderr, "SX11Window::bitfont called without beginDoubleBuffer\n");
+    return;
   }
-  if (gcFG != (unsigned long) _pen.getForeground().getValue())
+  if (dbuffer[dbindex].gcFD != (unsigned long) native)
   {
-    XSetForeground (impl->display, gc,
+    XSetFont (impl->display, dbuffer[dbindex].gc, (Font) native);
+    dbuffer[dbindex].gcFD  = (unsigned long) native;
+  }
+  if (dbuffer[dbindex].gcFG != (unsigned long) _pen.getForeground().getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc,
        SX11Color(impl, _pen.getForeground()).getPixelValue());
-    gcFG = (unsigned long) _pen.getForeground().getValue();
+    dbuffer[dbindex].gcFG = (unsigned long) _pen.getForeground().getValue();
   }
-  XDrawString16 (impl->display, (Window) id, gc, (int)x, (int)y, 
+  XDrawString16 (impl->display, (Drawable) dbuffer[dbindex].id, 
+    dbuffer[dbindex].gc, (int)x, (int)y, 
     (XChar2b*) data, len/2);
 }
 
 void
 SX11Window::setParent (SWindow* w, int x, int y)
 {
+  //fprintf (stderr, "reparent %lx\n", (unsigned long) this);
   SX11Window* x11w = (SX11Window*) w;
-  XReparentWindow (impl->display, (Window) id, (Window) x11w->id, x, y);
+  XReparentWindow (impl->display, (Window) dbuffer[0].id, (Window) 
+      x11w->dbuffer[0].id, x, y);
   isChild  = true;
 }
 
@@ -772,9 +838,15 @@ SX11Window::setParent (SWindow* w, int x, int y)
 void
 SX11Window::clear (int x, int y, unsigned int width, unsigned int height)
 {
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::clear called without beginDoubleBuffer\n");
+    return;
+  }
   bitfill (background, x, y, width, height);
 /* This one does not takes clip */
 /*
+  // does not work with pixmaps
   XClearArea (impl->display, (Window) id, x, y, width, height, 0);
 */
 }
@@ -792,9 +864,15 @@ void
 SX11Window::copy (int x, int y, unsigned int width, unsigned int height, 
   int tox, int toy)
 {
-  XCopyArea (impl->display, (Window) id, (Window) id, 
-      gc, x, y, width, height, tox, toy);
-  impl->moveRedrawEvent (id, tox-x, toy-y);
+  if (dbuffer[dbindex].id == None)
+  {
+    fprintf (stderr, "SX11Window::copy called without beginDoubleBuffer\n");
+    return;
+  }
+  XCopyArea (impl->display, (Drawable) dbuffer[dbindex].id, 
+     (Drawable) dbuffer[dbindex].id, 
+      dbuffer[dbindex].gc, x, y, width, height, tox, toy);
+  impl->moveRedrawEvent (dbuffer[0].id, tox-x, toy-y);
 }
 
 /**
@@ -808,12 +886,19 @@ void
 SX11Window::bitfill (const SColor& bg, int x, int y, 
  unsigned int width, unsigned int height)
 {
-  if (gcFG != (unsigned long)  bg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, bg).getPixelValue());
-    gcFG = (unsigned long)  bg.getValue();
+    fprintf (stderr, "SX11Window::bitfill called without beginDoubleBuffer\n");
+    return;
   }
-  XFillRectangle (impl->display, (Window) id, gc, x, y, width, height);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  bg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+       SX11Color(impl, bg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  bg.getValue();
+  }
+  XFillRectangle (impl->display, (Drawable) dbuffer[dbindex].id, 
+       dbuffer[dbindex].gc, x, y, width, height);
 }
 
 /**
@@ -826,12 +911,18 @@ SX11Window::bitfill (const SColor& bg, int x, int y,
 void
 SX11Window::bitline (const SColor& fg, int x, int y, int tox, int toy)
 {
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitline called without beginDoubleBuffer\n");
+    return;
   }
-  XDrawLine (impl->display, (Window) id, gc, x, y, tox, toy);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+         SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
+  }
+  XDrawLine (impl->display, (Drawable) dbuffer[dbindex].id, dbuffer[dbindex].gc, x, y, tox, toy);
 }
 
 /**
@@ -842,12 +933,17 @@ SX11Window::bitline (const SColor& fg, int x, int y, int tox, int toy)
 void
 SX11Window::bitpoint (const SColor& fg, int x, int y)
 {
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitpoint called without beginDoubleBuffer\n");
+    return;
   }
-  XDrawPoint (impl->display, (Window) id, gc, x, y);
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
+  }
+  XDrawPoint (impl->display, (Drawable) dbuffer[dbindex].id, dbuffer[dbindex].gc, x, y);
 }
 
 void
@@ -855,10 +951,16 @@ SX11Window::bitpoints (const SColor& fg, const int* _x, const int* _y,
          unsigned int _size)
 {
   if (_size==0) return;
-  if (gcFG != (unsigned long)  fg.getValue())
+  if (dbuffer[dbindex].id == None)
   {
-    XSetForeground (impl->display, gc, SX11Color(impl, fg).getPixelValue());
-    gcFG = (unsigned long)  fg.getValue();
+    fprintf (stderr, "SX11Window::bitpoints called without beginDoubleBuffer\n");
+    return;
+  }
+  if (dbuffer[dbindex].gcFG != (unsigned long)  fg.getValue())
+  {
+    XSetForeground (impl->display, dbuffer[dbindex].gc, 
+       SX11Color(impl, fg).getPixelValue());
+    dbuffer[dbindex].gcFG = (unsigned long)  fg.getValue();
     // If I don't do XSync there is a point drawing error of 
     // highlighted text on Matrox Video card with X64
     // XSync (impl->display, False);
@@ -870,7 +972,8 @@ SX11Window::bitpoints (const SColor& fg, const int* _x, const int* _y,
      p[i].x = _x[i];
      p[i].y = _y[i];
   }
-  XDrawPoints (impl->display, (Window) id, gc, p, _size, CoordModeOrigin);
+  XDrawPoints (impl->display, (Drawable) dbuffer[dbindex].id, 
+      dbuffer[dbindex].gc, p, _size, CoordModeOrigin);
   delete p;
 }
 
@@ -879,7 +982,8 @@ void
 SX11Window::getKeyboardFocus()
 {
   impl->getKeyboardFocus (this);
-  XSetInputFocus (impl->display, (Window) id, RevertToNone, CurrentTime);
+  XSetInputFocus (impl->display, (Window) dbuffer[0].id, 
+      RevertToNone, CurrentTime);
   XIC xic = inputMethod.getIC();
   if (xic)
   {
@@ -891,7 +995,7 @@ SX11Window::getKeyboardFocus()
 long
 SX11Window::getID ()
 {
-  return id;
+  return dbuffer[0].id;
 }
 
 /**
@@ -900,11 +1004,17 @@ SX11Window::getID ()
 void
 SX11Window::setClippingArea (int _x, int _y, unsigned int _width, unsigned int _height)
 {
+  // called outside dbufer begin and end
+  if (dbuffer[dbindex].id == None)
+  {
+    clipRegion = SX11Region (_x, _y, _width, _height);
+    return;
+  }
   XRectangle rect;
   rect.x = _x; rect.y = _y; rect.width = _width; rect.height = _height;
-  XSetClipRectangles (impl->display, gc, 0, 0, &rect, 1, Unsorted);
-  XSetClipRectangles (impl->display, gc_draw_and, 0, 0, &rect, 1, Unsorted);
-  XSetClipRectangles (impl->display, gc_draw_or, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc_draw_and, 0, 0, &rect, 1, Unsorted);
+  XSetClipRectangles (impl->display, dbuffer[dbindex].gc_draw_or, 0, 0, &rect, 1, Unsorted);
   clipped = true;
 }
 
@@ -914,11 +1024,14 @@ SX11Window::setClippingArea (int _x, int _y, unsigned int _width, unsigned int _
 void
 SX11Window::removeClippingArea ()
 {
+  // called outside dbufer begin and end
+  if (dbuffer[dbindex].id == None) return;
+
   if (clipped)
   {
-     XSetClipMask (impl->display, gc, None);
-     XSetClipMask (impl->display, gc_draw_and, None);
-     XSetClipMask (impl->display, gc_draw_or, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc_draw_and, None);
+     XSetClipMask (impl->display, dbuffer[dbindex].gc_draw_or, None);
      clipped = false;
   }
 }
@@ -940,7 +1053,10 @@ SX11Window::setApplicationImage (const SImage& im)
 {
   Pixmap pixmap = 0;
   Pixmap mask = 0;
+  unsigned int dbindexSave = dbindex;
+  dbindex = 0;
   createMaskedPixmap (im, &pixmap, &mask); 
+  dbindex = dbindexSave;
   //XWMHints *hints = XGetWMHints(impl->display, (Window) id);
   XWMHints *hints = XAllocWMHints();
   if (!pixmap ||!mask || !hints)
@@ -951,7 +1067,7 @@ SX11Window::setApplicationImage (const SImage& im)
   hints->icon_pixmap = pixmap;
   hints->flags |= IconMaskHint;
   hints->icon_mask = mask;
-  XSetWMHints(impl->display, (Window) id, hints); 
+  XSetWMHints(impl->display, (Window) dbuffer[0].id, hints); 
   XFree (hints);
 }
 
@@ -975,7 +1091,8 @@ SX11Window::redraw (bool clear, int _x, int _y, unsigned int _width, unsigned in
   {
     return;
   }
-  impl->addRedrawEvent (id, SRedrawEvent (clear, _x, _y, _width, _height)); 
+  impl->addRedrawEvent (dbuffer[0].id, 
+      SRedrawEvent (clear, _x, _y, _width, _height)); 
 }
 
 /**
@@ -1016,6 +1133,11 @@ SX11Window::getInputMethod ()
   return SString(inputMethod.getName());
 }
 
+bool
+SX11Window::hasStatusArea () {
+  return inputMethod.hasStatusArea ();
+}
+
 bool
 SX11Window::isVisible ()
 {
@@ -1072,7 +1194,7 @@ SX11Window::bitpolygon (const SColor& fg, const SV_INT& poligon)
 SString
 SX11Window::getClipUTF8()
 {
-  return impl->getClipUTF8 (id);
+  return impl->getClipUTF8 (dbuffer[0].id);
 }
 
 /**
@@ -1081,7 +1203,7 @@ SX11Window::getClipUTF8()
 void
 SX11Window::putClipUTF8(const SString& utf8)
 {
-  impl->putClipUTF8 (id, utf8);
+  impl->putClipUTF8 (dbuffer[0].id, utf8);
 }
 
 /**
@@ -1095,7 +1217,7 @@ SX11Window::setMinimumSize (unsigned int _width, unsigned int _height)
   sizeHints.min_width = _width;
   sizeHints.min_height = _height;
   sizeHints.flags = PMinSize;
-  XSetWMProperties (impl->display, (Window) id, 0, 0, 0, 0, &sizeHints, 0, 0);
+  XSetWMProperties (impl->display, (Window) dbuffer[0].id, 0, 0, 0, 0, &sizeHints, 0, 0);
 }
 
 void
@@ -1107,14 +1229,19 @@ SX11Window::setTitle (const SString& title)
   XTextProperty   propertyIconName;
   XTextProperty   propertyWindowName;
 
-  XStoreName (impl->display, (Window) id, windowName.array());
+  XStoreName (impl->display, (Window) dbuffer[0].id, windowName.array());
 
   /* Discard const */
   char* wnc = (char*) windowName.array();
+#ifdef X_HAVE_UTF8_STRING
+  if (Xutf8TextListToTextProperty (impl->display, &wnc, 1, XUTF8StringStyle, &propertyWindowName)==Success &&
+     Xutf8TextListToTextProperty (impl->display, &wnc, 1, XUTF8StringStyle, &propertyIconName)==Success)
+#else
   if (XStringListToTextProperty (&wnc, 1, &propertyWindowName)!=0 &&
      XStringListToTextProperty (&wnc, 1, &propertyIconName)!=0)
+#endif
   {
-    XSetWMProperties (impl->display, (Window) id,
+    XSetWMProperties (impl->display, (Window) dbuffer[0].id,
        &propertyWindowName, &propertyIconName,
         0, 0, // argv, argc,
         0, 0, 0);
@@ -1122,7 +1249,7 @@ SX11Window::setTitle (const SString& title)
         XFree (propertyIconName.value);
   }
 
-  XSetWMProtocols (impl->display, (Window) id, &impl->wmDeleteWindow, 1);
+  XSetWMProtocols (impl->display, (Window) dbuffer[0].id, &impl->wmDeleteWindow, 1);
 }
 
 /**
@@ -1141,14 +1268,14 @@ void
 SX11Window::addAccelerator (const SAccelerator& a, SAcceleratorListener* l)
 {
   accelerators.put (a.toString(), l);
-  impl->addAccelerator (id, a);
+  impl->addAccelerator (dbuffer[0].id, a);
 }
 
 void
 SX11Window::removeAccelerator (const SAccelerator& a, SAcceleratorListener* l)
 {
   accelerators.remove (a.toString());
-  impl->removeAccelerator (id, a);
+  impl->removeAccelerator (dbuffer[0].id, a);
 }
 
 void
@@ -1181,7 +1308,7 @@ void
 SX11Window::setDroppable (const SStringVector& _targets)
 {
   const Atom xdndVersion = 3;
-  XChangeProperty (impl->display, (Window) id, 
+  XChangeProperty (impl->display, (Window) dbuffer[0].id, 
      XInternAtom (impl->display, "XdndAware", False), XA_ATOM, 32,
      PropModeReplace, (unsigned char*) &xdndVersion, 1);
   for (unsigned int i=0; i<_targets.size(); i++)
@@ -1207,15 +1334,15 @@ SX11Window::setModal (SWindow* _parent, bool decorated)
   winattr.save_under = True;
   if (_parent == 0)
   {
-    XSetTransientForHint (impl->display, (Window) id, (Window)impl->root);
+    XSetTransientForHint (impl->display, (Window) dbuffer[0].id, (Window)impl->root);
   }
   else
   {
-    XSetTransientForHint (impl->display, (Window) id, (Window)
-     (((SX11Window*)_parent)->id));
+    XSetTransientForHint (impl->display, (Window) dbuffer[0].id, (Window)
+     (((SX11Window*)_parent)->dbuffer[0].id));
   }
   if (decorated) return;
-  XChangeWindowAttributes (impl->display, (Window) id,
+  XChangeWindowAttributes (impl->display, (Window) dbuffer[0].id,
        CWOverrideRedirect | CWSaveUnder, &winattr);
 }
 /**
@@ -1227,7 +1354,7 @@ SX11Window::center (SWindow* _window)
   Window cw = impl->root;
   if (_window!=0)
   {
-    cw = (Window) ((SX11Window*)_window)->id;
+    cw = (Window) ((SX11Window*)_window)->dbuffer[0].id;
     cw = impl->getTopLevelWindow (cw);
     
   }
@@ -1313,5 +1440,110 @@ SX11Window::isKInput ()
 unsigned long
 SX11Window::getWindowID() const
 {
-  return (unsigned long) id;
+  return (unsigned long) dbuffer[0].id;
+}
+
+void
+SX11Window::setDoubleBuffer (bool isOn)
+{
+  
+  dbindex = isOn ? 1 : 0;
+}
+
+bool
+SX11Window::isDoubleBufferEnabled () const
+{
+  return (dbindex == 1);
+}
+
+void
+SX11Window::freeDoubleBuffer ()
+{
+  if (dbuffer[1].id != None)
+  {
+    XFreeGC (impl->display, dbuffer[1].gc);
+    XFreeGC (impl->display, dbuffer[1].gc_default);
+    XFreeGC (impl->display, dbuffer[1].gc_draw_and);
+    XFreeGC (impl->display, dbuffer[1].gc_draw_or);
+    XFreePixmap (impl->display, (Pixmap) dbuffer[1].id);
+    dbuffer[1].id = None;
+  }
+}
+
+// called before global redraw to sync backing buffer with window.
+void
+SX11Window::beginDoubleBuffer (const SX11Region& region)
+{
+  if (dbindex == 0) return;
+  // Free Up Previous pixmap if 
+  if (dbuffer[1].id)
+  {
+    fprintf (stderr, "SX11Window::beginDoubleBuffer nested call?");
+    freeDoubleBuffer ();
+  }
+
+  dbuffer[1].gcFD = 0; // Font
+
+  unsigned int pw = (getWidth() == 0) ? 1 : getWidth();
+  unsigned int ph = (getHeight() == 0) ? 1 : getHeight();
+
+  int bits = DefaultDepth (impl->display, impl->screen);
+  dbuffer[1].id = XCreatePixmap (impl->display, (Window) dbuffer[0].id, 
+    pw,  ph, bits);
+
+  XGCValues   gcv;
+  gcv.foreground = SX11Color(impl, pen.getForeground()).getPixelValue();
+  gcv.background = background.getPixelValue();
+  clipped = false;
+  
+  dbuffer[1].gc = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground, &gcv); 
+  
+  dbuffer[1].gc_default = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+      0, &gcv);
+  
+  gcv.function = GXand;
+  dbuffer[1].gc_draw_and = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+    GCForeground | GCBackground | GCFunction, &gcv); 
+  
+  gcv.function = GXor;
+  dbuffer[1].gc_draw_or = XCreateGC (impl->display, (Drawable)dbuffer[0].id, 
+       GCForeground | GCBackground | GCFunction, &gcv); 
+
+  XSetForeground (impl->display,  dbuffer[1].gc, 
+     SX11Color(impl, background).getPixelValue());
+  dbuffer[1].gcFG = (unsigned long)  background.getValue();
+
+  XFillRectangle (impl->display, (Drawable) dbuffer[1].id, 
+      dbuffer[1].gc, region.x, region.y, region.width,  region.height);
+  if (clipped)
+  {
+    XRectangle rect;
+    rect.x = clipRegion.x;
+    rect.y = clipRegion.y;
+    rect.width = clipRegion.width;
+    rect.height = clipRegion.height;
+    XSetClipRectangles (impl->display, dbuffer[1].gc, 0, 0, &rect, 1, Unsorted);
+    XSetClipRectangles (impl->display, dbuffer[1].gc_draw_and, 0, 0, &rect, 1, Unsorted);
+    XSetClipRectangles (impl->display, dbuffer[1].gc_draw_or, 0, 0, &rect, 1, Unsorted);
+  }
+}
+
+void
+SX11Window::endDoubleBuffer (const SX11Region& region)
+{
+  if (dbindex == 0)
+  {
+     if (dbuffer[1].id != None) freeDoubleBuffer ();
+     return;
+  }
+  if (dbuffer[1].id == 0)
+  {
+    fprintf (stderr, "SX11Window::endDoubleBuffer nested call?");
+    return;
+  }
+  XCopyArea (impl->display, (Drawable) dbuffer[1].id, 
+    (Drawable)dbuffer[0].id, dbuffer[dbindex].gc_default, region.x, region.y, 
+    region.width, region.height, region.x, region.y);
+  freeDoubleBuffer ();
 }
